# -*- coding: utf-8 -*-

"""Base specific data provider."""
from typing import Any

from mimesis.providers import BaseDataProvider

__all__ = ["BaseSpecProvider"]


class BaseSpecProvider(BaseDataProvider):
    """Base provider for specific data providers."""

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        """Initialize attributes of superclass."""
        super().__init__(*args, **kwargs)
        self._datafile = "builtin.json"



# -*- coding: utf-8 -*-

"""Cryptographic data provider."""

import hashlib
import secrets
from typing import Any, Final, Optional
from uuid import UUID, uuid4

from mimesis.enums import Algorithm
from mimesis.locales import Locale
from mimesis.providers.base import BaseProvider
from mimesis.providers.text import Text

__all__ = ["Cryptographic"]


class Cryptographic(BaseProvider):
    """Class that provides cryptographic data."""

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        """Initialize attributes.

        :param seed: Seed.
        """
        super().__init__(*args, **kwargs)
        self._text = Text(locale=Locale.EN, seed=self.seed)

    class Meta:
        """Class for metadata."""

        name: Final[str] = "cryptographic"

    @staticmethod
    def uuid_object() -> UUID:
        """Generate UUID4 object.

        :return: UUID4 object.
        """
        return uuid4()

    def uuid(self) -> str:
        """Generate UUID4 string.

        :return: UUID4 as string.
        """
        return str(self.uuid_object())

    def hash(self, algorithm: Optional[Algorithm] = None) -> str:  # noqa: A003
        """Generate random hash.

        To change hashing algorithm, pass parameter ``algorithm``
        with needed value of the enum object :class:`~mimesis.enums.Algorithm`

        .. warning:: Seed is not applicable to this method,
            because of its cryptographic-safe nature.

        :param algorithm: Enum object :class:`~mimesis.enums.Algorithm`.
        :return: Hash.
        :raises NonEnumerableError: When algorithm is unsupported.
        """
        key = self.validate_enum(algorithm, Algorithm)

        if hasattr(hashlib, key):
            fn = getattr(hashlib, key)
            return fn(self.uuid().encode()).hexdigest()  # type: ignore

    @staticmethod
    def token_bytes(entropy: int = 32) -> bytes:
        """Generate byte string containing ``entropy`` bytes.

        The string has ``entropy`` random bytes, each byte
        converted to two hex digits.

        .. warning:: Seed is not applicable to this method,
            because of its cryptographic-safe nature.

        :param entropy: Number of bytes (default: 32).
        :return: Random bytes.
        """
        return secrets.token_bytes(entropy)

    @staticmethod
    def token_hex(entropy: int = 32) -> str:
        """Return a random text string, in hexadecimal.

        The string has *entropy* random bytes, each byte converted to two
        hex digits.  If *entropy* is ``None`` or not supplied, a reasonable
        default is used.

        .. warning:: Seed is not applicable to this method,
            because of its cryptographic-safe nature.

        :param entropy: Number of bytes (default: 32).
        :return: Token.
        """
        return secrets.token_hex(entropy)

    @staticmethod
    def token_urlsafe(entropy: int = 32) -> str:
        """Return a random URL-safe text string, in Base64 encoding.

        The string has *entropy* random bytes.  If *entropy* is ``None``
        or not supplied, a reasonable default is used.

        .. warning:: Seed is not applicable to this method,
            because of its cryptographic-safe nature.

        :param entropy: Number of bytes (default: 32).
        :return: URL-safe token.
        """
        return secrets.token_urlsafe(entropy)

    def mnemonic_phrase(self, length: int = 12, separator: Optional[str] = None) -> str:
        """Generate pseudo mnemonic phrase.

        Please, keep in mind that this method generates
        crypto-insecure values.

        :param separator: Separator of phrases (Default is " ").
        :param length: Number of words.
        :return: Mnemonic phrase.
        """
        if not separator:
            separator = " "

        words = self._text.words(quantity=length)
        return "{}".format(separator).join(words)



# -*- coding: utf-8 -*-

"""Base data provider."""

import contextlib
import functools
import json
import operator
from functools import reduce
from pathlib import Path
from typing import Any, Generator, List, Optional

from mimesis.exceptions import NonEnumerableError
from mimesis.locales import Locale, validate_locale
from mimesis.random import Random, get_random_item, random
from mimesis.typing import JSON, Seed

__all__ = ["BaseDataProvider", "BaseProvider"]


class BaseProvider:
    """This is a base class for all providers."""

    def __init__(self, *, seed: Optional[Seed] = None, **kwargs: Any) -> None:
        """Initialize attributes.

        Keep in mind, that locale-independent data providers will work
        only with keyword-only arguments since version 5.0.0.

        :param seed: Seed for random.
            When set to `None` the current system time is used.
        """
        self.seed = seed
        self.random = random

        if seed is not None:
            self.reseed(seed)

    def reseed(self, seed: Optional[Seed] = None) -> None:
        """Reseed the internal random generator.

        In case we use the default seed, we need to create a per instance
        random generator, in this case two providers with the same seed
        will always return the same values.

        :param seed: Seed for random.
            When set to `None` the current system time is used.
        """
        if self.random is random:
            self.random = Random()

        self.seed = seed
        self.random.seed(self.seed)

    def validate_enum(self, item: Any, enum: Any) -> Any:
        """Validate enum parameter of method in subclasses of BaseProvider.

        :param item: Item of enum object.
        :param enum: Enum object.
        :return: Value of item.
        :raises NonEnumerableError: if ``item`` not in ``enum``.
        """
        if item is None:
            result = get_random_item(enum, self.random)
        elif item and isinstance(item, enum):
            result = item
        else:
            raise NonEnumerableError(enum)

        return result.value

    def __str__(self) -> str:
        """Human-readable representation of locale."""
        return self.__class__.__name__


class BaseDataProvider(BaseProvider):
    """This is a base class for all data providers."""

    _LOCALE_SEPARATOR = "-"

    def __init__(
        self, locale: Locale = Locale.DEFAULT, seed: Optional[Seed] = None
    ) -> None:
        """Initialize attributes for data providers.

        :param locale: Current locale.
        :param seed: Seed to all the random functions.
        """
        super().__init__(seed=seed)
        self._data: JSON = {}
        self._datafile = ""
        self._setup_locale(locale)
        self._data_dir = Path(__file__).parent.parent.joinpath("data")

    def _setup_locale(self, locale: Locale = Locale.DEFAULT) -> None:
        """Set up locale after pre-check.

        :param str locale: Locale
        :raises UnsupportedLocale: When locale not supported.
        :return: Nothing.
        """

        locale_obj = validate_locale(locale)
        self.locale = locale_obj.value

    def extract(self, keys: List[str], default: Optional[Any] = None) -> Any:
        """Extracts nested values from JSON file by list of keys.

        :param keys: List of keys (order extremely matters).
        :param default: Default value.
        :return: Data.
        """

        if not keys:
            raise ValueError("List of keys cannot be empty.")

        try:
            if len(keys) == 1:
                return self._data[keys[0]]
            return reduce(operator.getitem, keys, self._data)
        except (TypeError, KeyError):
            return default

    def _update_dict(self, initial: JSON, other: JSON) -> JSON:
        """Recursively update a dictionary.

        :param initial: Dict to update.
        :param other: Dict to update from.
        :return: Updated dict.
        """
        for key, value in other.items():
            if isinstance(value, dict):
                r = self._update_dict(initial.get(key, {}), value)
                initial[key] = r
            else:
                initial[key] = other[key]
        return initial

    @functools.lru_cache(maxsize=None)
    def _load_datafile(self, datafile: str = "") -> None:
        """Pull the content from the JSON and memorize one.

        Opens JSON file ``file`` in the folder ``data/locale``
        and get content from the file and memorize ones using lru_cache.

        :param datafile: The name of file.
        :return: The content of the file.
        :raises UnsupportedLocale: Raises if locale is unsupported.
        """
        locale = self.locale
        data_dir = self._data_dir

        if not datafile:
            datafile = self._datafile

        def get_data(locale_name: str) -> Any:
            """Pull JSON data from file.

            :param locale_name: Locale name.
            :return: Content of JSON file as dict.
            """
            file_path = Path(data_dir).joinpath(locale_name, datafile)
            with open(file_path, "r", encoding="utf8") as f:
                return json.load(f)

        master_locale = locale.split(self._LOCALE_SEPARATOR).pop(0)
        data = get_data(master_locale)

        if self._LOCALE_SEPARATOR in locale:
            data = self._update_dict(data, get_data(locale))

        self._data = data

    def get_current_locale(self) -> str:
        """Get current locale.

        If locale is not defined then this method will always return ``en``,
        because ``en`` is default locale for all providers, excluding builtins.

        :return: Current locale.
        """
        return self.locale

    def _override_locale(self, locale: Locale = Locale.DEFAULT) -> None:
        """Overrides current locale with passed and pull data for new locale.

        :param locale: Locale
        :return: Nothing.
        """
        self._setup_locale(locale)
        self._load_datafile.cache_clear()
        self._load_datafile()

    @contextlib.contextmanager
    def override_locale(
        self,
        locale: Locale,
    ) -> Generator["BaseDataProvider", None, None]:
        """Context manager which allows overriding current locale.

        Temporarily overrides current locale for
        locale-dependent providers.

        :param locale: Locale.
        :return: Provider with overridden locale.
        """
        try:
            origin_locale = Locale(self.locale)
            self._override_locale(locale)
            try:
                yield self
            finally:
                self._override_locale(origin_locale)
        except AttributeError:
            raise ValueError(
                "«{}» has not locale dependent".format(self.__class__.__name__)
            )

    def __str__(self) -> str:
        """Human-readable representation of locale."""
        locale = getattr(self, "locale", Locale.DEFAULT)
        return "{} <{}>".format(self.__class__.__name__, Locale(locale))



# -*- coding: utf-8 -*-

"""Specific data provider for Denmark (da)."""
import operator
from typing import Final, Optional, Tuple

from mimesis import Datetime
from mimesis.builtins.base import BaseSpecProvider
from mimesis.locales import Locale
from mimesis.typing import Seed

__all__ = ["DenmarkSpecProvider"]


class DenmarkSpecProvider(BaseSpecProvider):
    """Class that provides special data for Denmark (da)."""

    def __init__(self, seed: Optional[Seed] = None) -> None:
        """Initialize attributes."""
        super().__init__(locale=Locale.DA, seed=seed)
        self._datetime = Datetime(locale=Locale.DA, seed=seed)
        self._checksum_factors = (4, 3, 2, 7, 6, 5, 4, 3, 2)

    class Meta:
        """The name of the provider."""

        name: Final[str] = "denmark_provider"

    def _calculate_century_selector(self, year: int) -> int:
        if 1858 <= year < 1900:
            return self.random.randint(5, 8)
        elif 1900 <= year < 1937:
            return self.random.randint(0, 3)
        elif 1937 <= year < 2000:
            return self.random.choice([4, 9])
        elif 2000 <= year < 2037:
            return self.random.randint(4, 9)

        raise ValueError("Invalid year")

    def _calculate_checksum(self, cpr_nr_no_checksum: str) -> int:
        """Calculate the CPR number checksum.

        The CPR checksum can be checked by:
        1. Multiplying each digit in the CPR number with a corresponding fixed
           factor (self._checksum_factors) to produce a list of products.
        2. Summing up all the products, including the checksum, and checking
           that the resulting sum modulo 11 is 0.

        As such the checksum can be determined by reordering the formula, as:
        * 11 - (sum_without_checksum % 11)

        If the sum_without_checksum is 0, the resulting checksum is 11, but
        returned as 0 according to the official rules.

        If the sum_without_checksum is 1, the resulting checksum is 10, and
        thus invalid as the checksum is only 1 digit, hence this implies that
        the generated serial_number is invalid.

        Note: This method does not handle checksum == 10 case.
              It is handled by recursion in _generate_serial_checksum.
        """
        cpr_digits = map(int, cpr_nr_no_checksum)
        cpr_digit_products = list(
            map(
                operator.mul,
                cpr_digits,
                self._checksum_factors,
            )
        )
        remainder = sum(cpr_digit_products) % 11
        if remainder == 0:
            return 0
        return 11 - remainder

    def _generate_serial_checksum(self, cpr_century: str) -> Tuple[str, int]:
        """Generate a serial number and checksum from cpr_century."""
        serial_number = "{:02d}".format(self.random.randint(0, 99))

        cpr_nr_no_checksum = "{}{}".format(cpr_century, serial_number)
        checksum = self._calculate_checksum(cpr_nr_no_checksum)
        if checksum == 10:
            return self._generate_serial_checksum(cpr_century)
        return serial_number, checksum

    def cpr(self) -> str:
        """Generate a random CPR number (Central Person Registry).

        :return: CPR number.

        :Example:
            0405420694
        """
        date = self._datetime.date(start=1858, end=2021)
        cpr_date = "{:02d}{:02d}{}".format(
            date.day,
            date.month,
            str(date.year)[-2:],
        )
        century_selector = self._calculate_century_selector(date.year)
        cpr_century = "{}{}".format(cpr_date, century_selector)
        serial_number, checksum = self._generate_serial_checksum(cpr_century)
        cpr_nr = "{}{}{}".format(cpr_century, serial_number, checksum)
        return cpr_nr



# -*- coding: utf-8 -*-

"""Specific data provider for USA (en)."""

from typing import Final, Optional

from mimesis.builtins.base import BaseSpecProvider
from mimesis.locales import Locale
from mimesis.typing import Seed

__all__ = ["USASpecProvider"]


class USASpecProvider(BaseSpecProvider):
    """Class that provides special data for USA (en)."""

    def __init__(self, seed: Optional[Seed] = None) -> None:
        """Initialize attributes."""
        super().__init__(locale=Locale.EN, seed=seed)

    class Meta:
        """The name of the provider."""

        name: Final[str] = "usa_provider"

    def tracking_number(self, service: str = "usps") -> str:
        """Generate random tracking number.

        Supported services: USPS, FedEx and UPS.

        :param str service: Post service.
        :return: Tracking number.
        """
        service = service.lower()

        if service not in ("usps", "fedex", "ups"):
            raise ValueError("Unsupported post service")

        services = {
            "usps": (
                "#### #### #### #### ####",
                "@@ ### ### ### US",
            ),
            "fedex": (
                "#### #### ####",
                "#### #### #### ###",
            ),
            "ups": ("1Z@####@##########",),
        }
        mask = self.random.choice(services[service])
        return self.random.custom_code(mask=mask)

    def ssn(self) -> str:
        """Generate a random, but valid SSN.

        :returns: SSN.

        :Example:
            569-66-5801
        """
        area = self.random.randint(1, 899)
        if area == 666:
            area = 665

        return "{:03}-{:02}-{:04}".format(
            area,
            self.random.randint(1, 99),
            self.random.randint(1, 9999),
        )



# -*- coding: utf-8 -*-

"""Provides a random choice from items in a sequence."""
import collections.abc
from typing import Any, Final, List, Optional, Sequence, Union

from mimesis.providers.base import BaseProvider

__all__ = ["Choice"]


class Choice(BaseProvider):
    """Class for generating a random choice from items in a sequence."""

    class Meta:
        """Class for metadata."""

        name: Final[str] = "choice"

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        """Initialize attributes.

        :param args: Arguments.
        :param kwargs: Keyword arguments.
        """
        super().__init__(*args, **kwargs)

    def __call__(
        self, items: Optional[Sequence[Any]], length: int = 0, unique: bool = False
    ) -> Union[Sequence[Any], Any]:
        """Generate a randomly-chosen sequence or bare element from a sequence.

        Provide elements randomly chosen from the elements in a sequence
        **items**, where when **length** is specified the random choices are
        contained in a sequence of the same type of length **length**,
        otherwise a single uncontained element is chosen. If **unique** is set
        to True, constrain a returned sequence to contain only unique elements.

        :param items: Non-empty sequence (list, tuple or string) of elements.
        :param length: Length of sequence (number of elements) to provide.
        :param unique: If True, ensures provided elements are unique.
        :return: Sequence or uncontained element randomly chosen from items.
        :raises TypeError: For non-sequence items or non-integer length.
        :raises ValueError: If negative length or insufficient unique elements.

        >>> from mimesis import Choice
        >>> choice = Choice()

        >>> choice(items=['a', 'b', 'c'])
        'c'
        >>> choice(items=['a', 'b', 'c'], length=1)
        ['a']
        >>> choice(items='abc', length=2)
        'ba'
        >>> choice(items=('a', 'b', 'c'), length=5)
        ('c', 'a', 'a', 'b', 'c')
        >>> choice(items='aabbbccccddddd', length=4, unique=True)
        'cdba'
        """

        if not isinstance(items, collections.abc.Sequence):
            raise TypeError("**items** must be non-empty sequence.")

        if not items:
            raise ValueError("**items** must be a non-empty sequence.")

        if length < 0:
            raise ValueError("**length** should be a positive integer.")

        if length == 0:
            return self.random.choice(items)

        data: List[str] = []
        if unique and len(set(items)) < length:  # avoid an infinite while loop
            raise ValueError(
                "There are not enough unique elements in "
                "**items** to provide the specified **number**."
            )
        while len(data) < length:
            item = self.random.choice(items)
            if (unique and item not in data) or not unique:
                data.append(item)

        if isinstance(items, list):
            return data
        elif isinstance(items, tuple):
            return tuple(data)
        return "".join(data)



# -*- coding: utf-8 -*-

"""Provides platforms."""

# https://docs.python.org/3.5/library/sys.html?highlight=sys#sys.platform
PLATFORMS = {
    "linux": {
        "home": "/home/",
    },
    "darwin": {
        "home": "/home/",
    },
    "win32": {
        "home": "C:\\Users\\",
    },
    "win64": {
        "home": "C:\\Users\\",
    },
}



# -*- coding: utf-8 -*-

"""Specific data provider for Netherlands (nl)."""

from typing import Final, Optional

from mimesis.builtins.base import BaseSpecProvider
from mimesis.locales import Locale
from mimesis.typing import Seed

__all__ = ["NetherlandsSpecProvider"]


class NetherlandsSpecProvider(BaseSpecProvider):
    """Class that provides special data for Netherlands (nl)."""

    def __init__(self, seed: Optional[Seed] = None) -> None:
        """Initialize attributes."""
        super().__init__(locale=Locale.NL, seed=seed)

    class Meta:
        """The name of the provider."""

        name: Final[str] = "netherlands_provider"

    def bsn(self) -> str:
        """Generate a random, but valid ``Burgerservicenummer``.

        :returns: Random BSN.

        :Example:
            255159705
        """

        def _is_valid_bsn(number: str) -> bool:
            total = 0
            multiplier = 9

            for char in number:
                multiplier = -multiplier if multiplier == 1 else multiplier
                total += int(char) * multiplier
                multiplier -= 1

            result = total % 11 == 0
            return result

        a, b = (100000000, 999999999)
        sample = str(self.random.randint(a, b))

        while not _is_valid_bsn(sample):
            sample = str(self.random.randint(a, b))

        return sample

    def burgerservicenummer(self) -> str:
        """Generate a random, but valid ``Burgerservicenummer``.

        An alias for self.bsn()
        """
        return self.bsn()



# -*- coding: utf-8 -*-

"""Provides data related to transports."""

from typing import Any, Final, Optional

from mimesis.data import (
    AIRPLANES,
    CARS,
    MANUFACTURERS,
    TRUCKS,
    VR_CODES,
    VRC_BY_LOCALES,
)
from mimesis.locales import Locale
from mimesis.providers.base import BaseProvider

__all__ = ["Transport"]


class Transport(BaseProvider):
    """Class for generating data related to transports."""

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        """Initialize attributes.

        :param locale: Current locale.
        :param seed: Seed.
        """
        super().__init__(*args, **kwargs)

    class Meta:
        """Class for metadata."""

        name: Final[str] = "transport"

    def truck(self, model_mask: str = "#### @@") -> str:
        """Generate a truck model.

        :param model_mask: Mask of truck model. Here '@' is a
            placeholder of characters and '#' is a placeholder of digits.
        :return: Dummy truck model.

        :Example:
            Caledon-966O.
        """
        return "{}-{}".format(
            self.random.choice(TRUCKS),
            self.random.custom_code(model_mask),
        )

    def manufacturer(self) -> str:
        """Get a random card manufacturer.

        :return: A car manufacturer

        :Example:
            Tesla.
        """
        return self.random.choice(MANUFACTURERS)

    def car(self) -> str:
        """Get a random vehicle.

        :return: A vehicle.

        :Example:
            Tesla Model S.
        """
        return self.random.choice(CARS)

    def airplane(self, model_mask: str = "###") -> str:
        """Generate a dummy airplane model.

        :param model_mask: Mask of truck model. Here '@' is a
            placeholder of characters and '#' is a placeholder of digits.
        :return: Airplane model.

        :Example:
            Boeing 727.
        """
        model = self.random.custom_code(mask=model_mask)
        plane = self.random.choice(AIRPLANES)
        return "{} {}".format(plane, model)

    def vehicle_registration_code(self, locale: Optional[Locale] = None) -> str:
        """Get vehicle registration code of country.

        :param locale: Registration code for locale (country).
        :return: Vehicle registration code.
        """
        if locale:
            return VRC_BY_LOCALES[locale.value]

        return self.random.choice(VR_CODES)



from .int.address import *
from .int.code import *
from .int.common import *
from .int.datetime import *
from .int.development import *
from .int.file import *
from .int.finance import *
from .int.hardware import *
from .int.internet import *
from .int.path import *
from .int.payment import *
from .int.person import *
from .int.scientific import *
from .int.text import *
from .int.transport import *



# -*- coding: utf-8 -*-

"""Provides all the data related to the files."""

EXTENSIONS = {
    "source": [
        ".a",
        ".asm",
        ".asp",
        ".awk",
        ".c",
        ".class",
        ".cpp",
        ".pl",
        ".js",
        ".java",
        ".clj",
        ".py",
        ".rb",
        ".hs",
        ".erl",
        ".rs",
        ".swift",
        ".html",
        ".json",
        ".xml",
        ".css",
        ".php",
        ".jl",
        ".r",
        ".cs",
        "d",
        ".lisp",
        ".cl",
        ".go",
        ".h",
        ".scala",
        ".sc",
        ".ts",
        ".sql",
    ],
    "text": [
        ".doc",
        ".docx",
        ".log",
        ".rtf",
        ".md",
        ".pdf",
        ".odt",
        ".txt",
    ],
    "data": [
        ".csv",
        ".dat",
        ".ged",
        ".pps",
        ".ppt",
        ".pptx",
    ],
    "audio": [
        ".flac",
        ".mp3",
        ".m3u",
        ".m4a",
        ".wav",
        ".wma",
    ],
    "video": [
        ".3gp",
        ".mp4",
        ".abi",
        ".m4v",
        ".mov",
        ".mpg",
        ".wmv",
    ],
    "image": [
        ".bmp",
        ".jpg",
        ".jpeg",
        ".png",
        ".svg",
    ],
    "executable": [
        ".apk",
        ".app",
        ".bat",
        ".jar",
        ".com",
        ".exe",
    ],
    "compressed": [
        ".7z",
        ".war",
        ".zip",
        ".tar.gz",
        ".tar.xz",
        ".rar",
    ],
}

MIME_TYPES = {
    "application": [
        "application/1d-interleaved-parityfec",
        "application/3gpdash-qoe-report+xml",
        "application/3gpp-ims+xml",
        "application/A2L",
        "application/activemessage",
        "application/alto-costmap+json",
        "application/alto-costmapfilter+json",
        "application/alto-directory+json",
        "application/alto-endpointcost+json",
        "application/alto-endpointcostparams+json",
        "application/alto-endpointprop+json",
        "application/alto-endpointpropparams+json",
        "application/alto-error+json",
        "application/alto-networkmap+json",
        "application/alto-networkmapfilter+json",
        "application/AML",
        "application/andrew-inset",
        "application/applefile",
        "application/ATF",
        "application/ATFX",
        "application/atom+xml",
        "application/atomcat+xml",
        "application/atomdeleted+xml",
        "application/atomicmail",
        "application/atomsvc+xml",
        "application/ATXML",
        "application/auth-policy+xml",
        "application/bacnet-xdd+zip",
        "application/batch-SMTP",
        "application/beep+xml",
        "application/calendar+json",
        "application/calendar+xml",
        "application/call-completion",
        "application/cals-1840",
        "application/cbor",
        "application/ccmp+xml",
        "application/ccxml+xml",
        "application/CDFX+XML",
        "application/cdmi-capability",
        "application/cdmi-container",
        "application/cdmi-domain",
        "application/cdmi-object",
        "application/cdmi-queue",
        "application/CEA",
        "application/cea-2018+xml",
        "application/cellml+xml",
        "application/cfw",
        "application/cms",
        "application/cnrp+xml",
        "application/coap-group+json",
        "application/commonground",
        "application/conference-info+xml",
        "application/cpl+xml",
        "application/csrattrs",
        "application/csta+xml",
        "application/CSTAdata+xml",
        "application/cybercash",
        "application/dash+xml",
        "application/dashdelta",
        "application/davmount+xml",
        "application/dca-rft",
        "application/DCD",
        "application/dec-dx",
        "application/dialog-info+xml",
        "application/dicom",
        "application/DII",
        "application/DIT",
        "application/dns",
        "application/dskpp+xml",
        "application/dssc+der",
        "application/dssc+xml",
        "application/dvcs",
        "application/ecmascript",
        "application/EDI-consent",
        "application/EDI-X12",
        "application/EDIFACT",
        "application/emma+xml",
        "application/emotionml+xml",
        "application/encaprtp",
        "application/epp+xml",
        "application/epub+zip",
        "application/eshop",
        "application/example",
        "application/exi",
        "application/fastinfoset",
        "application/fastsoap",
        "application/fdt+xml",
        "application/fits",
        "application/font-sfnt",
        "application/font-tdpfr",
        "application/font-woff",
        "application/framework-attributes+xml",
        "application/gzip",
        "application/H224",
        "application/held+xml",
        "application/http",
        "application/hyperstudio",
        "application/ibe-key-request+xml",
        "application/ibe-pkg-reply+xml",
        "application/ibe-pp-data",
        "application/iges",
        "application/im-iscomposing+xml",
        "application/index",
        "application/index.cmd",
        "application/index.obj",
        "application/index.response",
        "application/index.vnd",
        "application/inkml+xml",
        "application/iotp",
        "application/ipfix",
        "application/ipp",
        "application/isup",
        "application/its+xml",
        "application/javascript",
        "application/jose",
        "application/jose+json",
        "application/jrd+json",
        "application/json",
        "application/json-patch+json",
        "application/json-seq",
        "application/jwk-set+json",
        "application/jwk+json",
        "application/jwt",
        "application/kpml-request+xml",
        "application/kpml-response+xml",
        "application/ld+json",
        "application/link-format",
        "application/load-control+xml",
        "application/lost+xml",
        "application/lostsync+xml",
        "application/LXF",
        "application/mac-binhex40",
        "application/macwriteii",
        "application/mads+xml",
        "application/marc",
        "application/marcxml+xml",
        "application/mathematica",
        "application/mathml-content+xml",
        "application/mathml-presentation+xml",
        "application/mathml+xml",
        "application/mbms-associated-procedure-description+xml",
        "application/mbms-deregister+xml",
        "application/mbms-envelope+xml",
        "application/mbms-msk-response+xml",
        "application/mbms-msk+xml",
        "application/mbms-protection-description+xml",
        "application/mbms-reception-report+xml",
        "application/mbms-register-response+xml",
        "application/mbms-register+xml",
        "application/mbms-schedule+xml",
        "application/mbms-user-service-description+xml",
        "application/mbox",
        "application/media_control+xml",
        "application/media-policy-dataset+xml",
        "application/mediaservercontrol+xml",
        "application/merge-patch+json",
        "application/metalink4+xml",
        "application/mets+xml",
        "application/MF4",
        "application/mikey",
        "application/mods+xml",
        "application/moss-keys",
        "application/moss-signature",
        "application/mosskey-data",
        "application/mosskey-request",
        "application/mp21",
        "application/mp4",
        "application/mpeg4-generic",
        "application/mpeg4-iod",
        "application/mpeg4-iod-xmt",
        "application/mrb-consumer+xml",
        "application/mrb-publish+xml",
        "application/msc-ivr+xml",
        "application/msc-mixer+xml",
        "application/msword",
        "application/mxf",
        "application/nasdata",
        "application/news-checkgroups",
        "application/news-groupinfo",
        "application/news-transmission",
        "application/nlsml+xml",
        "application/nss",
        "application/ocsp-request",
        "application/ocsp-response",
        "application/octet-stream",
        "application/oda",
        "application/ODX",
        "application/oebps-package+xml",
        "application/ogg",
        "application/oxps",
        "application/p2p-overlay+xml",
        "application/parityfec",
        "application/patch-ops-error+xml",
        "application/pdf",
        "application/PDX",
        "application/pgp-encrypted",
        "application/pgp-keys",
        "application/pgp-signature",
        "application/pidf-diff+xml",
        "application/pidf+xml",
        "application/pkcs10",
        "application/pkcs7-mime",
        "application/pkcs7-signature",
        "application/pkcs8",
        "application/pkix-attr-cert",
        "application/pkix-cert",
        "application/pkix-crl",
        "application/pkix-pkipath",
        "application/pkixcmp",
        "application/pls+xml",
        "application/poc-settings+xml",
        "application/postscript",
        "application/provenance+xml",
        "application/prs.alvestrand.titrax-sheet",
        "application/prs.cww",
        "application/prs.hpub+zip",
        "application/prs.nprend",
        "application/prs.plucker",
        "application/prs.rdf-xml-crypt",
        "application/prs.xsf+xml",
        "application/pskc+xml",
        "application/qsig",
        "application/raptorfec",
        "application/rdap+json",
        "application/rdf+xml",
        "application/reginfo+xml",
        "application/relax-ng-compact-syntax",
        "application/remote-printing",
        "application/reputon+json",
        "application/resource-lists-diff+xml",
        "application/resource-lists+xml",
        "application/riscos",
        "application/rlmi+xml",
        "application/rls-services+xml",
        "application/rpki-ghostbusters",
        "application/rpki-manifest",
        "application/rpki-roa",
        "application/rpki-updown",
        "application/rtf",
        "application/rtploopback",
        "application/rtx",
        "application/samlassertion+xml",
        "application/samlmetadata+xml",
        "application/sbml+xml",
        "application/scaip+xml",
        "application/scvp-cv-request",
        "application/scvp-cv-response",
        "application/scvp-vp-request",
        "application/scvp-vp-response",
        "application/sdp",
        "application/sep-exi",
        "application/sep+xml",
        "application/session-info",
        "application/set-payment",
        "application/set-payment-initiation",
        "application/set-registration",
        "application/set-registration-initiation",
        "application/sgml",
        "application/sgml-open-catalog",
        "application/shf+xml",
        "application/sieve",
        "application/simple-filter+xml",
        "application/simple-message-summary",
        "application/simpleSymbolContainer",
        "application/slate",
        "application/smil - OBSOLETED in favor of application/smil+xml",
        "application/smil+xml",
        "application/smpte336m",
        "application/soap+fastinfoset",
        "application/soap+xml",
        "application/sparql-query",
        "application/sparql-results+xml",
        "application/spirits-event+xml",
        "application/sql",
        "application/srgs",
        "application/srgs+xml",
        "application/sru+xml",
        "application/ssml+xml",
        "application/tamp-apex-update",
        "application/tamp-apex-update-confirm",
        "application/tamp-community-update",
        "application/tamp-community-update-confirm",
        "application/tamp-error",
        "application/tamp-sequence-adjust",
        "application/tamp-sequence-adjust-confirm",
        "application/tamp-status-query",
        "application/tamp-status-response",
        "application/tamp-update",
        "application/tamp-update-confirm",
        "application/tei+xml",
        "application/thraud+xml",
        "application/timestamp-query",
        "application/timestamp-reply",
        "application/timestamped-data",
        "application/ttml+xml",
        "application/tve-trigger",
        "application/ulpfec",
        "application/urc-grpsheet+xml",
        "application/urc-ressheet+xml",
        "application/urc-targetdesc+xml",
        "application/urc-uisocketdesc+xml",
        "application/vcard+json",
        "application/vcard+xml",
        "application/vemmi",
        "application/vnd.3gpp.bsf+xml",
        "application/vnd.3gpp.pic-bw-large",
        "application/vnd.3gpp.pic-bw-small",
        "application/vnd.3gpp.pic-bw-var",
        "application/vnd.3gpp.sms",
        "application/vnd.3gpp2.bcmcsinfo+xml",
        "application/vnd.3gpp2.sms",
        "application/vnd.3gpp2.tcap",
        "application/vnd.3M.Post-it-Notes",
        "application/vnd.accpac.simply.aso",
        "application/vnd.accpac.simply.imp",
        "application/vnd.acucobol",
        "application/vnd.acucorp",
        "application/vnd.adobe.flash.movie",
        "application/vnd.adobe.formscentral.fcdt",
        "application/vnd.adobe.fxp",
        "application/vnd.adobe.partial-upload",
        "application/vnd.adobe.xdp+xml",
        "application/vnd.adobe.xfdf",
        "application/vnd.aether.imp",
        "application/vnd.ah-barcode",
        "application/vnd.ahead.space",
        "application/vnd.airzip.filesecure.azf",
        "application/vnd.airzip.filesecure.azs",
        "application/vnd.americandynamics.acc",
        "application/vnd.amiga.ami",
        "application/vnd.amundsen.maze+xml",
        "application/vnd.anser-web-certificate-issue-initiation",
        "application/vnd.antix.game-component",
        "application/vnd.apache.thrift.binary",
        "application/vnd.apache.thrift.compact",
        "application/vnd.apache.thrift.json",
        "application/vnd.api+json",
        "application/vnd.apple.installer+xml",
        "application/vnd.apple.mpegurl",
        "application/vnd.aristanetworks.swi",
        "application/vnd.artsquare",
        "application/vnd.astraea-software.iota",
        "application/vnd.audiograph",
        "application/vnd.autopackage",
        "application/vnd.avistar+xml",
        "application/vnd.balsamiq.bmml+xml",
        "application/vnd.bekitzur-stech+json",
        "application/vnd.blueice.multipass",
        "application/vnd.bluetooth.ep.oob",
        "application/vnd.bluetooth.le.oob",
        "application/vnd.bmi",
        "application/vnd.businessobjects",
        "application/vnd.cab-jscript",
        "application/vnd.canon-cpdl",
        "application/vnd.canon-lips",
        "application/vnd.cendio.thinlinc.clientconf",
        "application/vnd.century-systems.tcp_stream",
        "application/vnd.chemdraw+xml",
        "application/vnd.chipnuts.karaoke-mmd",
        "application/vnd.cinderella",
        "application/vnd.cirpack.isdn-ext",
        "application/vnd.citationstyles.style+xml",
        "application/vnd.claymore",
        "application/vnd.cloanto.rp9",
        "application/vnd.clonk.c4group",
        "application/vnd.cluetrust.cartomobile-config",
        "application/vnd.cluetrust.cartomobile-config-pkg",
        "application/vnd.coffeescript",
        "application/vnd.collection.doc+json",
        "application/vnd.collection.next+json",
        "application/vnd.collection+json",
        "application/vnd.commerce-battelle",
        "application/vnd.commonspace",
        "application/vnd.contact.cmsg",
        "application/vnd.cosmocaller",
        "application/vnd.crick.clicker",
        "application/vnd.crick.clicker.keyboard",
        "application/vnd.crick.clicker.palette",
        "application/vnd.crick.clicker.template",
        "application/vnd.crick.clicker.wordbank",
        "application/vnd.criticaltools.wbs+xml",
        "application/vnd.ctc-posml",
        "application/vnd.ctct.ws+xml",
        "application/vnd.cups-pdf",
        "application/vnd.cups-postscript",
        "application/vnd.cups-ppd",
        "application/vnd.cups-raster",
        "application/vnd.cups-raw",
        "application/vnd.curl",
        "application/vnd.cyan.dean.root+xml",
        "application/vnd.cybank",
        "application/vnd.dart",
        "application/vnd.data-vision.rdz",
        "application/vnd.debian.binary-package",
        "application/vnd.dece.data",
        "application/vnd.dece.ttml+xml",
        "application/vnd.dece.unspecified",
        "application/vnd.dece.zip",
        "application/vnd.denovo.fcselayout-link",
        "application/vnd.desmume.movie",
        "application/vnd.dir-bi.plate-dl-nosuffix",
        "application/vnd.dm.delegation+xml",
        "application/vnd.dna_sequence",
        "application/vnd.document+json",
        "application/vnd.dolby.mobile.1",
        "application/vnd.dolby.mobile.2",
        "application/vnd.doremir.scorecloud-binary-document",
        "application/vnd.dpgraph",
        "application/vnd.dreamfactory",
        "application/vnd.dtg.local",
        "application/vnd.dtg.local.flash",
        "application/vnd.dtg.local.html",
        "application/vnd.dvb.ait",
        "application/vnd.dvb.dvbj",
        "application/vnd.dvb.esgcontainer",
        "application/vnd.dvb.ipdcdftnotifaccess",
        "application/vnd.dvb.ipdcesgaccess",
        "application/vnd.dvb.ipdcesgaccess2",
        "application/vnd.dvb.ipdcesgpdd",
        "application/vnd.dvb.ipdcroaming",
        "application/vnd.dvb.iptv.alfec-base",
        "application/vnd.dvb.iptv.alfec-enhancement",
        "application/vnd.dvb.notif-aggregate-root+xml",
        "application/vnd.dvb.notif-container+xml",
        "application/vnd.dvb.notif-generic+xml",
        "application/vnd.dvb.notif-ia-msglist+xml",
        "application/vnd.dvb.notif-ia-registration-request+xml",
        "application/vnd.dvb.notif-ia-registration-response+xml",
        "application/vnd.dvb.notif-init+xml",
        "application/vnd.dvb.pfr",
        "application/vnd.dvb.service",
        "application/vnd.dxr",
        "application/vnd.dynageo",
        "application/vnd.dzr",
        "application/vnd.easykaraoke.cdgdownload",
        "application/vnd.ecdis-update",
        "application/vnd.ecowin.chart",
        "application/vnd.ecowin.filerequest",
        "application/vnd.ecowin.fileupdate",
        "application/vnd.ecowin.series",
        "application/vnd.ecowin.seriesrequest",
        "application/vnd.ecowin.seriesupdate",
        "application/vnd.emclient.accessrequest+xml",
        "application/vnd.enliven",
        "application/vnd.enphase.envoy",
        "application/vnd.eprints.data+xml",
        "application/vnd.epson.esf",
        "application/vnd.epson.msf",
        "application/vnd.epson.quickanime",
        "application/vnd.epson.salt",
        "application/vnd.epson.ssf",
        "application/vnd.ericsson.quickcall",
        "application/vnd.eszigno3+xml",
        "application/vnd.etsi.aoc+xml",
        "application/vnd.etsi.asic-e+zip",
        "application/vnd.etsi.asic-s+zip",
        "application/vnd.etsi.cug+xml",
        "application/vnd.etsi.iptvcommand+xml",
        "application/vnd.etsi.iptvdiscovery+xml",
        "application/vnd.etsi.iptvprofile+xml",
        "application/vnd.etsi.iptvsad-bc+xml",
        "application/vnd.etsi.iptvsad-cod+xml",
        "application/vnd.etsi.iptvsad-npvr+xml",
        "application/vnd.etsi.iptvservice+xml",
        "application/vnd.etsi.iptvsync+xml",
        "application/vnd.etsi.iptvueprofile+xml",
        "application/vnd.etsi.mcid+xml",
        "application/vnd.etsi.mheg5",
        "application/vnd.etsi.overload-control-policy-dataset+xml",
        "application/vnd.etsi.pstn+xml",
        "application/vnd.etsi.sci+xml",
        "application/vnd.etsi.simservs+xml",
        "application/vnd.etsi.timestamp-token",
        "application/vnd.etsi.tsl.der",
        "application/vnd.etsi.tsl+xml",
        "application/vnd.eudora.data",
        "application/vnd.ezpix-album",
        "application/vnd.ezpix-package",
        "application/vnd.f-secure.mobile",
        "application/vnd.fdf",
        "application/vnd.fdsn.mseed",
        "application/vnd.fdsn.seed",
        "application/vnd.ffsns",
        "application/vnd.fints",
        "application/vnd.FloGraphIt",
        "application/vnd.fluxtime.clip",
        "application/vnd.font-fontforge-sfd",
        "application/vnd.framemaker",
        "application/vnd.frogans.fnc",
        "application/vnd.frogans.ltf",
        "application/vnd.fsc.weblaunch",
        "application/vnd.fujitsu.oasys",
        "application/vnd.fujitsu.oasys2",
        "application/vnd.fujitsu.oasys3",
        "application/vnd.fujitsu.oasysgp",
        "application/vnd.fujitsu.oasysprs",
        "application/vnd.fujixerox.ART-EX",
        "application/vnd.fujixerox.ART4",
        "application/vnd.fujixerox.ddd",
        "application/vnd.fujixerox.docuworks",
        "application/vnd.fujixerox.docuworks.binder",
        "application/vnd.fujixerox.docuworks.container",
        "application/vnd.fujixerox.HBPL",
        "application/vnd.fut-misnet",
        "application/vnd.fuzzysheet",
        "application/vnd.genomatix.tuxedo",
        "application/vnd.geo+json",
        "application/vnd.geocube+xml - OBSOLETED by request",
        "application/vnd.geogebra.file",
        "application/vnd.geogebra.tool",
        "application/vnd.geometry-explorer",
        "application/vnd.geonext",
        "application/vnd.geoplan",
        "application/vnd.geospace",
        "application/vnd.gerber",
        "application/vnd.globalplatform.card-content-mgt",
        "application/vnd.globalplatform.card-content-mgt-response",
        "application/vnd.gmx - DEPRECATED",
        "application/vnd.google-earth.kml+xml",
        "application/vnd.google-earth.kmz",
        "application/vnd.gov.sk.e-form+xml",
        "application/vnd.gov.sk.e-form+zip",
        "application/vnd.gov.sk.xmldatacontainer+xml",
        "application/vnd.grafeq",
        "application/vnd.gridmp",
        "application/vnd.groove-account",
        "application/vnd.groove-help",
        "application/vnd.groove-identity-message",
        "application/vnd.groove-injector",
        "application/vnd.groove-tool-message",
        "application/vnd.groove-tool-template",
        "application/vnd.groove-vcard",
        "application/vnd.hal+json",
        "application/vnd.hal+xml",
        "application/vnd.HandHeld-Entertainment+xml",
        "application/vnd.hbci",
        "application/vnd.hcl-bireports",
        "application/vnd.heroku+json",
        "application/vnd.hhe.lesson-player",
        "application/vnd.hp-HPGL",
        "application/vnd.hp-hpid",
        "application/vnd.hp-hps",
        "application/vnd.hp-jlyt",
        "application/vnd.hp-PCL",
        "application/vnd.hp-PCLXL",
        "application/vnd.httphone",
        "application/vnd.hydrostatix.sof-data",
        "application/vnd.hzn-3d-crossword",
        "application/vnd.ibm.afplinedata",
        "application/vnd.ibm.electronic-media",
        "application/vnd.ibm.MiniPay",
        "application/vnd.ibm.modcap",
        "application/vnd.ibm.rights-management",
        "application/vnd.ibm.secure-container",
        "application/vnd.iccprofile",
        "application/vnd.ieee.1905",
        "application/vnd.igloader",
        "application/vnd.immervision-ivp",
        "application/vnd.immervision-ivu",
        "application/vnd.ims.imsccv1p1",
        "application/vnd.ims.imsccv1p2",
        "application/vnd.ims.imsccv1p3",
        "application/vnd.ims.lis.v2.result+json",
        "application/vnd.ims.lti.v2.toolconsumerprofile+json",
        "application/vnd.ims.lti.v2.toolproxy.id+json",
        "application/vnd.ims.lti.v2.toolproxy+json",
        "application/vnd.ims.lti.v2.toolsettings.simple+json",
        "application/vnd.ims.lti.v2.toolsettings+json",
        "application/vnd.informedcontrol.rms+xml",
        "application/vnd.informix-visionary",
        "application/vnd.infotech.project",
        "application/vnd.infotech.project+xml",
        "application/vnd.innopath.wamp.notification",
        "application/vnd.insors.igm",
        "application/vnd.intercon.formnet",
        "application/vnd.intergeo",
        "application/vnd.intertrust.digibox",
        "application/vnd.intertrust.nncp",
        "application/vnd.intu.qbo",
        "application/vnd.intu.qfx",
        "application/vnd.iptc.g2.catalogitem+xml",
        "application/vnd.iptc.g2.conceptitem+xml",
        "application/vnd.iptc.g2.knowledgeitem+xml",
        "application/vnd.iptc.g2.newsitem+xml",
        "application/vnd.iptc.g2.newsmessage+xml",
        "application/vnd.iptc.g2.packageitem+xml",
        "application/vnd.iptc.g2.planningitem+xml",
        "application/vnd.ipunplugged.rcprofile",
        "application/vnd.irepository.package+xml",
        "application/vnd.is-xpr",
        "application/vnd.isac.fcs",
        "application/vnd.jam",
        "application/vnd.japannet-directory-service",
        "application/vnd.japannet-jpnstore-wakeup",
        "application/vnd.japannet-payment-wakeup",
        "application/vnd.japannet-registration",
        "application/vnd.japannet-registration-wakeup",
        "application/vnd.japannet-setstore-wakeup",
        "application/vnd.japannet-verification",
        "application/vnd.japannet-verification-wakeup",
        "application/vnd.jcp.javame.midlet-rms",
        "application/vnd.jisp",
        "application/vnd.joost.joda-archive",
        "application/vnd.jsk.isdn-ngn",
        "application/vnd.kahootz",
        "application/vnd.kde.karbon",
        "application/vnd.kde.kchart",
        "application/vnd.kde.kformula",
        "application/vnd.kde.kivio",
        "application/vnd.kde.kontour",
        "application/vnd.kde.kpresenter",
        "application/vnd.kde.kspread",
        "application/vnd.kde.kword",
        "application/vnd.kenameaapp",
        "application/vnd.kidspiration",
        "application/vnd.Kinar",
        "application/vnd.koan",
        "application/vnd.kodak-descriptor",
        "application/vnd.las.las+xml",
        "application/vnd.liberty-request+xml",
        "application/vnd.llamagraphics.life-balance.desktop",
        "application/vnd.llamagraphics.life-balance.exchange+xml",
        "application/vnd.lotus-1-2-3",
        "application/vnd.lotus-approach",
        "application/vnd.lotus-freelance",
        "application/vnd.lotus-notes",
        "application/vnd.lotus-organizer",
        "application/vnd.lotus-screencam",
        "application/vnd.lotus-wordpro",
        "application/vnd.macports.portpkg",
        "application/vnd.marlin.drm.actiontoken+xml",
        "application/vnd.marlin.drm.conftoken+xml",
        "application/vnd.marlin.drm.license+xml",
        "application/vnd.marlin.drm.mdcf",
        "application/vnd.mason+json",
        "application/vnd.maxmind.maxmind-db",
        "application/vnd.mcd",
        "application/vnd.medcalcdata",
        "application/vnd.mediastation.cdkey",
        "application/vnd.meridian-slingshot",
        "application/vnd.MFER",
        "application/vnd.mfmp",
        "application/vnd.micro+json",
        "application/vnd.micrografx.flo",
        "application/vnd.micrografx.igx",
        "application/vnd.microsoft.portable-executable",
        "application/vnd.miele+json",
        "application/vnd.mif",
        "application/vnd.minisoft-hp3000-save",
        "application/vnd.mitsubishi.misty-guard.trustweb",
        "application/vnd.Mobius.DAF",
        "application/vnd.Mobius.DIS",
        "application/vnd.Mobius.MBK",
        "application/vnd.Mobius.MQY",
        "application/vnd.Mobius.MSL",
        "application/vnd.Mobius.PLC",
        "application/vnd.Mobius.TXF",
        "application/vnd.mophun.application",
        "application/vnd.mophun.certificate",
        "application/vnd.motorola.flexsuite",
        "application/vnd.motorola.flexsuite.adsi",
        "application/vnd.motorola.flexsuite.fis",
        "application/vnd.motorola.flexsuite.gotap",
        "application/vnd.motorola.flexsuite.kmr",
        "application/vnd.motorola.flexsuite.ttc",
        "application/vnd.motorola.flexsuite.wem",
        "application/vnd.motorola.iprm",
        "application/vnd.mozilla.xul+xml",
        "application/vnd.ms-3mfdocument",
        "application/vnd.ms-artgalry",
        "application/vnd.ms-asf",
        "application/vnd.ms-cab-compressed",
        "application/vnd.ms-excel",
        "application/vnd.ms-excel.addin.macroEnabled.12",
        "application/vnd.ms-excel.sheet.binary.macroEnabled.12",
        "application/vnd.ms-excel.sheet.macroEnabled.12",
        "application/vnd.ms-excel.template.macroEnabled.12",
        "application/vnd.ms-fontobject",
        "application/vnd.ms-htmlhelp",
        "application/vnd.ms-ims",
        "application/vnd.ms-lrm",
        "application/vnd.ms-office.activeX+xml",
        "application/vnd.ms-officetheme",
        "application/vnd.ms-playready.initiator+xml",
        "application/vnd.ms-powerpoint",
        "application/vnd.ms-powerpoint.addin.macroEnabled.12",
        "application/vnd.ms-powerpoint.presentation.macroEnabled.12",
        "application/vnd.ms-powerpoint.slide.macroEnabled.12",
        "application/vnd.ms-powerpoint.slideshow.macroEnabled.12",
        "application/vnd.ms-powerpoint.template.macroEnabled.12",
        "application/vnd.ms-project",
        "application/vnd.ms-tnef",
        "application/vnd.ms-windows.printerpairing",
        "application/vnd.ms-wmdrm.lic-chlg-req",
        "application/vnd.ms-wmdrm.lic-resp",
        "application/vnd.ms-wmdrm.meter-chlg-req",
        "application/vnd.ms-wmdrm.meter-resp",
        "application/vnd.ms-word.document.macroEnabled.12",
        "application/vnd.ms-word.template.macroEnabled.12",
        "application/vnd.ms-works",
        "application/vnd.ms-wpl",
        "application/vnd.ms-xpsdocument",
        "application/vnd.msa-disk-image",
        "application/vnd.mseq",
        "application/vnd.msign",
        "application/vnd.multiad.creator",
        "application/vnd.multiad.creator.cif",
        "application/vnd.music-niff",
        "application/vnd.musician",
        "application/vnd.muvee.style",
        "application/vnd.mynfc",
        "application/vnd.ncd.control",
        "application/vnd.ncd.reference",
        "application/vnd.nervana",
        "application/vnd.netfpx",
        "application/vnd.neurolanguage.nlu",
        "application/vnd.nintendo.nitro.rom",
        "application/vnd.nintendo.snes.rom",
        "application/vnd.nitf",
        "application/vnd.noblenet-directory",
        "application/vnd.noblenet-sealer",
        "application/vnd.noblenet-web",
        "application/vnd.nokia.catalogs",
        "application/vnd.nokia.conml+wbxml",
        "application/vnd.nokia.conml+xml",
        "application/vnd.nokia.iptv.config+xml",
        "application/vnd.nokia.iSDS-radio-presets",
        "application/vnd.nokia.landmark+wbxml",
        "application/vnd.nokia.landmark+xml",
        "application/vnd.nokia.landmarkcollection+xml",
        "application/vnd.nokia.n-gage.ac+xml",
        "application/vnd.nokia.n-gage.data",
        "application/vnd.nokia.n-gage.symbian.install",
        "application/vnd.nokia.ncd",
        "application/vnd.nokia.pcd+wbxml",
        "application/vnd.nokia.pcd+xml",
        "application/vnd.nokia.radio-preset",
        "application/vnd.nokia.radio-presets",
        "application/vnd.novadigm.EDM",
        "application/vnd.novadigm.EDX",
        "application/vnd.novadigm.EXT",
        "application/vnd.ntt-local.content-share",
        "application/vnd.ntt-local.file-transfer",
        "application/vnd.ntt-local.ogw_remote-access",
        "application/vnd.ntt-local.sip-ta_remote",
        "application/vnd.ntt-local.sip-ta_tcp_stream",
        "application/vnd.oasis.opendocument.chart",
        "application/vnd.oasis.opendocument.chart-template",
        "application/vnd.oasis.opendocument.database",
        "application/vnd.oasis.opendocument.formula",
        "application/vnd.oasis.opendocument.formula-template",
        "application/vnd.oasis.opendocument.graphics",
        "application/vnd.oasis.opendocument.graphics-template",
        "application/vnd.oasis.opendocument.image",
        "application/vnd.oasis.opendocument.image-template",
        "application/vnd.oasis.opendocument.presentation",
        "application/vnd.oasis.opendocument.presentation-template",
        "application/vnd.oasis.opendocument.spreadsheet",
        "application/vnd.oasis.opendocument.spreadsheet-template",
        "application/vnd.oasis.opendocument.text",
        "application/vnd.oasis.opendocument.text-master",
        "application/vnd.oasis.opendocument.text-template",
        "application/vnd.oasis.opendocument.text-web",
        "application/vnd.obn",
        "application/vnd.oftn.l10n+json",
        "application/vnd.oipf.contentaccessdownload+xml",
        "application/vnd.oipf.contentaccessstreaming+xml",
        "application/vnd.oipf.cspg-hexbinary",
        "application/vnd.oipf.dae.svg+xml",
        "application/vnd.oipf.dae.xhtml+xml",
        "application/vnd.oipf.mippvcontrolmessage+xml",
        "application/vnd.oipf.pae.gem",
        "application/vnd.oipf.spdiscovery+xml",
        "application/vnd.oipf.spdlist+xml",
        "application/vnd.oipf.ueprofile+xml",
        "application/vnd.oipf.userprofile+xml",
        "application/vnd.olpc-sugar",
        "application/vnd.oma-scws-config",
        "application/vnd.oma-scws-http-request",
        "application/vnd.oma-scws-http-response",
        "application/vnd.oma.bcast.associated-procedure-parameter+xml",
        "application/vnd.oma.bcast.drm-trigger+xml",
        "application/vnd.oma.bcast.imd+xml",
        "application/vnd.oma.bcast.ltkm",
        "application/vnd.oma.bcast.notification+xml",
        "application/vnd.oma.bcast.provisioningtrigger",
        "application/vnd.oma.bcast.sgboot",
        "application/vnd.oma.bcast.sgdd+xml",
        "application/vnd.oma.bcast.sgdu",
        "application/vnd.oma.bcast.simple-symbol-container",
        "application/vnd.oma.bcast.smartcard-trigger+xml",
        "application/vnd.oma.bcast.sprov+xml",
        "application/vnd.oma.bcast.stkm",
        "application/vnd.oma.cab-address-book+xml",
        "application/vnd.oma.cab-feature-handler+xml",
        "application/vnd.oma.cab-pcc+xml",
        "application/vnd.oma.cab-subs-invite+xml",
        "application/vnd.oma.cab-user-prefs+xml",
        "application/vnd.oma.dcd",
        "application/vnd.oma.dcdc",
        "application/vnd.oma.dd2+xml",
        "application/vnd.oma.drm.risd+xml",
        "application/vnd.oma.group-usage-list+xml",
        "application/vnd.oma.pal+xml",
        "application/vnd.oma.poc.detailed-progress-report+xml",
        "application/vnd.oma.poc.final-report+xml",
        "application/vnd.oma.poc.groups+xml",
        "application/vnd.oma.poc.invocation-descriptor+xml",
        "application/vnd.oma.poc.optimized-progress-report+xml",
        "application/vnd.oma.push",
        "application/vnd.oma.scidm.messages+xml",
        "application/vnd.oma.xcap-directory+xml",
        "application/vnd.omads-email+xml",
        "application/vnd.omads-file+xml",
        "application/vnd.omads-folder+xml",
        "application/vnd.omaloc-supl-init",
        "application/vnd.openeye.oeb",
        "application/vnd.oracle.resource+json",
        "application/vnd.orange.indata",
        "application/vnd.osa.netdeploy",
        "application/vnd.osgeo.mapguide.package",
        "application/vnd.osgi.bundle",
        "application/vnd.osgi.dp",
        "application/vnd.osgi.subsystem",
        "application/vnd.otps.ct-kip+xml",
        "application/vnd.palm",
        "application/vnd.panoply",
        "application/vnd.paos.xml",
        "application/vnd.pawaafile",
        "application/vnd.pcos",
        "application/vnd.pg.format",
        "application/vnd.pg.osasli",
        "application/vnd.piaccess.application-licence",
        "application/vnd.picsel",
        "application/vnd.pmi.widget",
        "application/vnd.poc.group-advertisement+xml",
        "application/vnd.pocketlearn",
        "application/vnd.powerbuilder6",
        "application/vnd.powerbuilder6-s",
        "application/vnd.powerbuilder7",
        "application/vnd.powerbuilder7-s",
        "application/vnd.powerbuilder75",
        "application/vnd.powerbuilder75-s",
        "application/vnd.preminet",
        "application/vnd.previewsystems.box",
        "application/vnd.proteus.magazine",
        "application/vnd.publishare-delta-tree",
        "application/vnd.pvi.ptid1",
        "application/vnd.pwg-multiplexed",
        "application/vnd.pwg-xhtml-print+xml",
        "application/vnd.qualcomm.brew-app-res",
        "application/vnd.Quark.QuarkXPress",
        "application/vnd.quobject-quoxdocument",
        "application/vnd.radisys.moml+xml",
        "application/vnd.radisys.msml-audit-conf+xml",
        "application/vnd.radisys.msml-audit-conn+xml",
        "application/vnd.radisys.msml-audit-dialog+xml",
        "application/vnd.radisys.msml-audit-stream+xml",
        "application/vnd.radisys.msml-audit+xml",
        "application/vnd.radisys.msml-conf+xml",
        "application/vnd.radisys.msml-dialog-base+xml",
        "application/vnd.radisys.msml-dialog-fax-detect+xml",
        "application/vnd.radisys.msml-dialog-fax-sendrecv+xml",
        "application/vnd.radisys.msml-dialog-group+xml",
        "application/vnd.radisys.msml-dialog-speech+xml",
        "application/vnd.radisys.msml-dialog-transform+xml",
        "application/vnd.radisys.msml-dialog+xml",
        "application/vnd.radisys.msml+xml",
        "application/vnd.rainstor.data",
        "application/vnd.rapid",
        "application/vnd.realvnc.bed",
        "application/vnd.recordare.musicxml",
        "application/vnd.recordare.musicxml+xml",
        "application/vnd.RenLearn.rlprint",
        "application/vnd.rig.cryptonote",
        "application/vnd.route66.link66+xml",
        "application/vnd.rs-274x",
        "application/vnd.ruckus.download",
        "application/vnd.s3sms",
        "application/vnd.sailingtracker.track",
        "application/vnd.sbm.cid",
        "application/vnd.sbm.mid2",
        "application/vnd.scribus",
        "application/vnd.sealed.3df",
        "application/vnd.sealed.csf",
        "application/vnd.sealed.doc",
        "application/vnd.sealed.eml",
        "application/vnd.sealed.mht",
        "application/vnd.sealed.net",
        "application/vnd.sealed.ppt",
        "application/vnd.sealed.tiff",
        "application/vnd.sealed.xls",
        "application/vnd.sealedmedia.softseal.html",
        "application/vnd.sealedmedia.softseal.pdf",
        "application/vnd.seemail",
        "application/vnd.sema",
        "application/vnd.semd",
        "application/vnd.semf",
        "application/vnd.shana.informed.formdata",
        "application/vnd.shana.informed.formtemplate",
        "application/vnd.shana.informed.interchange",
        "application/vnd.shana.informed.package",
        "application/vnd.SimTech-MindMapper",
        "application/vnd.siren+json",
        "application/vnd.smaf",
        "application/vnd.smart.notebook",
        "application/vnd.smart.teacher",
        "application/vnd.software602.filler.form-xml-zip",
        "application/vnd.software602.filler.form+xml",
        "application/vnd.solent.sdkm+xml",
        "application/vnd.spotfire.dxp",
        "application/vnd.spotfire.sfs",
        "application/vnd.sss-cod",
        "application/vnd.sss-dtf",
        "application/vnd.sss-ntf",
        "application/vnd.stepmania.package",
        "application/vnd.stepmania.stepchart",
        "application/vnd.street-stream",
        "application/vnd.sun.wadl+xml",
        "application/vnd.sus-calendar",
        "application/vnd.svd",
        "application/vnd.swiftview-ics",
        "application/vnd.syncml.dm.notification",
        "application/vnd.syncml.dm+wbxml",
        "application/vnd.syncml.dm+xml",
        "application/vnd.syncml.dmddf+wbxml",
        "application/vnd.syncml.dmddf+xml",
        "application/vnd.syncml.dmtnds+wbxml",
        "application/vnd.syncml.dmtnds+xml",
        "application/vnd.syncml.ds.notification",
        "application/vnd.syncml+xml",
        "application/vnd.tao.intent-module-archive",
        "application/vnd.tcpdump.pcap",
        "application/vnd.tmd.mediaflex.api+xml",
        "application/vnd.tmobile-livetv",
        "application/vnd.trid.tpt",
        "application/vnd.triscape.mxs",
        "application/vnd.trueapp",
        "application/vnd.truedoc",
        "application/vnd.ubisoft.webplayer",
        "application/vnd.ufdl",
        "application/vnd.uiq.theme",
        "application/vnd.umajin",
        "application/vnd.unity",
        "application/vnd.uoml+xml",
        "application/vnd.uplanet.alert",
        "application/vnd.uplanet.alert-wbxml",
        "application/vnd.uplanet.bearer-choice",
        "application/vnd.uplanet.bearer-choice-wbxml",
        "application/vnd.uplanet.cacheop",
        "application/vnd.uplanet.cacheop-wbxml",
        "application/vnd.uplanet.channel",
        "application/vnd.uplanet.channel-wbxml",
        "application/vnd.uplanet.list",
        "application/vnd.uplanet.list-wbxml",
        "application/vnd.uplanet.listcmd",
        "application/vnd.uplanet.listcmd-wbxml",
        "application/vnd.uplanet.signal",
        "application/vnd.valve.source.material",
        "application/vnd.vcx",
        "application/vnd.vd-study",
        "application/vnd.vectorworks",
        "application/vnd.verimatrix.vcas",
        "application/vnd.vidsoft.vidconference",
        "application/vnd.visio",
        "application/vnd.visionary",
        "application/vnd.vividence.scriptfile",
        "application/vnd.vsf",
        "application/vnd.wap.sic",
        "application/vnd.wap.slc",
        "application/vnd.wap.wbxml",
        "application/vnd.wap.wmlc",
        "application/vnd.wap.wmlscriptc",
        "application/vnd.webturbo",
        "application/vnd.wfa.p2p",
        "application/vnd.wfa.wsc",
        "application/vnd.windows.devicepairing",
        "application/vnd.wmc",
        "application/vnd.wmf.bootstrap",
        "application/vnd.wolfram.mathematica",
        "application/vnd.wolfram.mathematica.package",
        "application/vnd.wolfram.player",
        "application/vnd.wordperfect",
        "application/vnd.wqd",
        "application/vnd.wrq-hp3000-labelled",
        "application/vnd.wt.stf",
        "application/vnd.wv.csp+wbxml",
        "application/vnd.wv.csp+xml",
        "application/vnd.wv.ssp+xml",
        "application/vnd.xacml+json",
        "application/vnd.xara",
        "application/vnd.xfdl",
        "application/vnd.xfdl.webform",
        "application/vnd.xmi+xml",
        "application/vnd.xmpie.cpkg",
        "application/vnd.xmpie.dpkg",
        "application/vnd.xmpie.plan",
        "application/vnd.xmpie.ppkg",
        "application/vnd.xmpie.xlim",
        "application/vnd.yamaha.hv-dic",
        "application/vnd.yamaha.hv-script",
        "application/vnd.yamaha.hv-voice",
        "application/vnd.yamaha.openscoreformat",
        "application/vnd.yamaha.openscoreformat.osfpvg+xml",
        "application/vnd.yamaha.remote-setup",
        "application/vnd.yamaha.smaf-audio",
        "application/vnd.yamaha.smaf-phrase",
        "application/vnd.yamaha.through-ngn",
        "application/vnd.yamaha.tunnel-udpencap",
        "application/vnd.yaoweme",
        "application/vnd.yellowriver-custom-menu",
        "application/vnd.zul",
        "application/vnd.zzazz.deck+xml",
        "application/voicexml+xml",
        "application/vq-rtcpxr",
        "application/watcherinfo+xml",
        "application/whoispp-query",
        "application/whoispp-response",
        "application/widget",
        "application/wita",
        "application/wordperfect5.1",
        "application/wsdl+xml",
        "application/wspolicy+xml",
        "application/x-www-form-urlencoded",
        "application/x400-bp",
        "application/xacml+xml",
        "application/xcap-att+xml",
        "application/xcap-caps+xml",
        "application/xcap-diff+xml",
        "application/xcap-el+xml",
        "application/xcap-error+xml",
        "application/xcap-ns+xml",
        "application/xcon-conference-info-diff+xml",
        "application/xcon-conference-info+xml",
        "application/xenc+xml",
        "application/xhtml-voice+xml",
        "application/xhtml+xml",
        "application/xml",
        "application/xml-dtd",
        "application/xml-external-parsed-entity",
        "application/xml-patch+xml",
        "application/xmpp+xml",
        "application/xop+xml",
        "application/xslt+xml",
        "application/xv+xml",
        "application/yang",
        "application/yin+xml",
        "application/zip",
        "application/zlib",
    ],
    "audio": [
        "audio/1d-interleaved-parityfec",
        "audio/32kadpcm",
        "audio/3gpp",
        "audio/3gpp2",
        "audio/ac3",
        "audio/AMR",
        "audio/AMR-WB",
        "audio/amr-wb+",
        "audio/aptx",
        "audio/asc",
        "audio/ATRAC-ADVANCED-LOSSLESS",
        "audio/ATRAC-X",
        "audio/ATRAC3",
        "audio/basic",
        "audio/BV16",
        "audio/BV32",
        "audio/clearmode",
        "audio/CN",
        "audio/DAT12",
        "audio/dls",
        "audio/dsr-es201108",
        "audio/dsr-es202050",
        "audio/dsr-es202211",
        "audio/dsr-es202212",
        "audio/DV",
        "audio/DVI4",
        "audio/eac3",
        "audio/encaprtp",
        "audio/EVRC",
        "audio/EVRC-QCP",
        "audio/EVRC0",
        "audio/EVRC1",
        "audio/EVRCB",
        "audio/EVRCB0",
        "audio/EVRCB1",
        "audio/EVRCNW",
        "audio/EVRCNW0",
        "audio/EVRCNW1",
        "audio/EVRCWB",
        "audio/EVRCWB0",
        "audio/EVRCWB1",
        "audio/example",
        "audio/fwdred",
        "audio/G719",
        "audio/G722",
        "audio/G7221",
        "audio/G723",
        "audio/G726-16",
        "audio/G726-24",
        "audio/G726-32",
        "audio/G726-40",
        "audio/G728",
        "audio/G729",
        "audio/G7291",
        "audio/G729D",
        "audio/G729E",
        "audio/GSM",
        "audio/GSM-EFR",
        "audio/GSM-HR-08",
        "audio/iLBC",
        "audio/ip-mr_v2.5",
        "audio/L16",
        "audio/L20",
        "audio/L24",
        "audio/L8",
        "audio/LPC",
        "audio/mobile-xmf",
        "audio/mp4",
        "audio/MP4A-LATM",
        "audio/MPA",
        "audio/mpa-robust",
        "audio/mpeg",
        "audio/mpeg4-generic",
        "audio/ogg",
        "audio/opus",
        "audio/parityfec",
        "audio/PCMA",
        "audio/PCMA-WB",
        "audio/PCMU",
        "audio/PCMU-WB",
        "audio/prs.sid",
        "audio/QCELP",
        "audio/raptorfec",
        "audio/RED",
        "audio/rtp-enc-aescm128",
        "audio/rtp-midi",
        "audio/rtploopback",
        "audio/rtx",
        "audio/SMV",
        "audio/SMV-QCP",
        "audio/SMV0",
        "audio/sp-midi",
        "audio/speex",
        "audio/t140c",
        "audio/t38",
        "audio/telephone-event",
        "audio/tone",
        "audio/UEMCLIP",
        "audio/ulpfec",
        "audio/VDVI",
        "audio/VMR-WB",
        "audio/vnd.3gpp.iufp",
        "audio/vnd.4SB",
        "audio/vnd.audiokoz",
        "audio/vnd.CELP",
        "audio/vnd.cisco.nse",
        "audio/vnd.cmles.radio-events",
        "audio/vnd.cns.anp1",
        "audio/vnd.cns.inf1",
        "audio/vnd.dece.audio",
        "audio/vnd.digital-winds",
        "audio/vnd.dlna.adts",
        "audio/vnd.dolby.heaac.1",
        "audio/vnd.dolby.heaac.2",
        "audio/vnd.dolby.mlp",
        "audio/vnd.dolby.mps",
        "audio/vnd.dolby.pl2",
        "audio/vnd.dolby.pl2x",
        "audio/vnd.dolby.pl2z",
        "audio/vnd.dolby.pulse.1",
        "audio/vnd.dra",
        "audio/vnd.dts",
        "audio/vnd.dts.hd",
        "audio/vnd.dvb.file",
        "audio/vnd.everad.plj",
        "audio/vnd.hns.audio",
        "audio/vnd.lucent.voice",
        "audio/vnd.ms-playready.media.pya",
        "audio/vnd.nokia.mobile-xmf",
        "audio/vnd.nortel.vbk",
        "audio/vnd.nuera.ecelp4800",
        "audio/vnd.nuera.ecelp7470",
        "audio/vnd.nuera.ecelp9600",
        "audio/vnd.octel.sbc",
        "audio/vnd.qcelp",
        "audio/vnd.rhetorex.32kadpcm",
        "audio/vnd.rip",
        "audio/vnd.sealedmedia.softseal.mpeg",
        "audio/vnd.vmx.cvsd",
        "audio/vorbis",
        "audio/vorbis-config",
    ],
    "image": [
        "image/cgm",
        "image/example",
        "image/fits",
        "image/g3fax",
        "image/gif",
        "image/ief",
        "image/jp2",
        "image/jpeg",
        "image/jpm",
        "image/jpx",
        "image/ktx",
        "image/naplps",
        "image/png",
        "image/prs.btif",
        "image/prs.pti",
        "image/pwg-raster",
        "image/svg+xml",
        "image/t38",
        "image/tiff",
        "image/tiff-fx",
        "image/vnd.adobe.photoshop",
        "image/vnd.airzip.accelerator.azv",
        "image/vnd.cns.inf2",
        "image/vnd.dece.graphic",
        "image/vnd.djvu",
        "image/vnd.dvb.subtitle",
        "image/vnd.dwg",
        "image/vnd.dxf",
        "image/vnd.fastbidsheet",
        "image/vnd.fpx",
        "image/vnd.fst",
        "image/vnd.fujixerox.edmics-mmr",
        "image/vnd.fujixerox.edmics-rlc",
        "image/vnd.globalgraphics.pgb",
        "image/vnd.microsoft.icon",
        "image/vnd.mix",
        "image/vnd.ms-modi",
        "image/vnd.net-fpx",
        "image/vnd.radiance",
        "image/vnd.sealed.png",
        "image/vnd.sealedmedia.softseal.gif",
        "image/vnd.sealedmedia.softseal.jpg",
        "image/vnd.svf",
        "image/vnd.tencent.tap",
        "image/vnd.valve.source.texture",
        "image/vnd.wap.wbmp",
        "image/vnd.xiff",
        "image/vnd.zbrush.pcx",
    ],
    "message": [
        "message/CPIM",
        "message/delivery-status",
        "message/disposition-notification",
        "message/example",
        "message/external-body",
        "message/feedback-report",
        "message/global",
        "message/global-delivery-status",
        "message/global-disposition-notification",
        "message/global-headers",
        "message/http",
        "message/imdn+xml",
        "message/news",
        "message/partial",
        "message/rfc822",
        "message/s-http",
        "message/sip",
        "message/sipfrag",
        "message/tracking-status",
        "message/vnd.si.simp",
        "message/vnd.wfa.wsc",
    ],
    "text": [
        "text/1d-interleaved-parityfec",
        "text/cache-manifest",
        "text/calendar",
        "text/css",
        "text/csv",
        "text/csv-schema",
        "text/dns",
        "text/ecmascript",
        "text/encaprtp",
        "text/enriched",
        "text/example",
        "text/fwdred",
        "text/grammar-ref-list",
        "text/html",
        "text/javascript",
        "text/jcr-cnd",
        "text/mizar",
        "text/n3",
        "text/parameters",
        "text/parityfec",
        "text/plain",
        "text/provenance-notation",
        "text/prs.fallenstein.rst",
        "text/prs.lines.tag",
        "text/raptorfec",
        "text/RED",
        "text/rfc822-headers",
        "text/richtext",
        "text/rtf",
        "text/rtp-enc-aescm128",
        "text/rtploopback",
        "text/rtx",
        "text/sgml",
        "text/t140",
        "text/tab-separated-values",
        "text/troff",
        "text/turtle",
        "text/ulpfec",
        "text/uri-list",
        "text/vcard",
        "text/vnd.a",
        "text/vnd.abc",
        "text/vnd.curl",
        "text/vnd.debian.copyright",
        "text/vnd.DMClientScript",
        "text/vnd.dvb.subtitle",
        "text/vnd.esmertec.theme-descriptor",
        "text/vnd.fly",
        "text/vnd.fmi.flexstor",
        "text/vnd.graphviz",
        "text/vnd.in3d.3dml",
        "text/vnd.in3d.spot",
        "text/vnd.IPTC.NewsML",
        "text/vnd.IPTC.NITF",
        "text/vnd.latex-z",
        "text/vnd.motorola.reflex",
        "text/vnd.ms-mediapackage",
        "text/vnd.net2phone.commcenter.command",
        "text/vnd.radisys.msml-basic-layout",
        "text/vnd.si.uricatalogue - OBSOLETED by request",
        "text/vnd.sun.j2me.app-descriptor",
        "text/vnd.trolltech.linguist",
        "text/vnd.wap.si",
        "text/vnd.wap.sl",
        "text/vnd.wap.wml",
        "text/vnd.wap.wmlscript",
        "text/xml",
        "text/xml-external-parsed-entity",
    ],
    "video": [
        "video/1d-interleaved-parityfec",
        "video/3gpp",
        "video/3gpp-tt",
        "video/3gpp2",
        "video/BMPEG",
        "video/BT656",
        "video/CelB",
        "video/DV",
        "video/encaprtp",
        "video/example",
        "video/H261",
        "video/H263",
        "video/H263-1998",
        "video/H263-2000",
        "video/H264",
        "video/H264-RCDO",
        "video/H264-SVC",
        "video/iso.segment",
        "video/JPEG",
        "video/jpeg2000",
        "video/mj2",
        "video/MP1S",
        "video/MP2P",
        "video/MP2T",
        "video/mp4",
        "video/MP4V-ES",
        "video/mpeg",
        "video/mpeg4-generic",
        "video/MPV",
        "video/nv",
        "video/ogg",
        "video/parityfec",
        "video/pointer",
        "video/quicktime",
        "video/raptorfec",
        "video/raw",
        "video/rtp-enc-aescm128",
        "video/rtploopback",
        "video/rtx",
        "video/SMPTE292M",
        "video/ulpfec",
        "video/vc1",
        "video/vnd.CCTV",
        "video/vnd.dece.hd",
        "video/vnd.dece.mobile",
        "video/vnd.dece.mp4",
        "video/vnd.dece.pd",
        "video/vnd.dece.sd",
        "video/vnd.dece.video",
        "video/vnd.directv.mpeg",
        "video/vnd.directv.mpeg-tts",
        "video/vnd.dlna.mpeg-tts",
        "video/vnd.dvb.file",
        "video/vnd.fvt",
        "video/vnd.hns.video",
        "video/vnd.iptvforum.1dparityfec-1010",
        "video/vnd.iptvforum.1dparityfec-2005",
        "video/vnd.iptvforum.2dparityfec-1010",
        "video/vnd.iptvforum.2dparityfec-2005",
        "video/vnd.iptvforum.ttsavc",
        "video/vnd.iptvforum.ttsmpeg2",
        "video/vnd.motorola.video",
        "video/vnd.motorola.videop",
        "video/vnd.mpegurl",
        "video/vnd.ms-playready.media.pyv",
        "video/vnd.nokia.interleaved-multimedia",
        "video/vnd.nokia.videovoip",
        "video/vnd.objectvideo",
        "video/vnd.radgamettools.bink",
        "video/vnd.radgamettools.smacker",
        "video/vnd.sealed.mpeg1",
        "video/vnd.sealed.mpeg4",
        "video/vnd.sealed.swf",
        "video/vnd.sealedmedia.softseal.mov",
        "video/vnd.uvvu.mp4",
        "video/vnd.vivo",
    ],
}



# -*- coding: utf-8 -*-

"""Provides all at one."""

import inspect
from typing import Any, Final, List, Optional, Type

from mimesis.locales import Locale
from mimesis.providers.address import Address
from mimesis.providers.base import BaseDataProvider, BaseProvider
from mimesis.providers.binaryfile import BinaryFile
from mimesis.providers.choice import Choice
from mimesis.providers.code import Code
from mimesis.providers.cryptographic import Cryptographic
from mimesis.providers.date import Datetime
from mimesis.providers.development import Development
from mimesis.providers.file import File
from mimesis.providers.finance import Finance
from mimesis.providers.food import Food
from mimesis.providers.hardware import Hardware
from mimesis.providers.internet import Internet
from mimesis.providers.numeric import Numeric
from mimesis.providers.path import Path
from mimesis.providers.payment import Payment
from mimesis.providers.person import Person
from mimesis.providers.science import Science
from mimesis.providers.text import Text
from mimesis.providers.transport import Transport
from mimesis.typing import Seed

__all__ = ["Generic"]


class Generic(BaseProvider):
    """Class which contain all providers at one."""

    _DEFAULT_PROVIDERS = (
        Address,
        BinaryFile,
        Finance,
        Choice,
        Code,
        Choice,
        Datetime,
        Development,
        File,
        Food,
        Hardware,
        Internet,
        Numeric,
        Path,
        Payment,
        Person,
        Science,
        Text,
        Transport,
        Cryptographic,
    )

    def __init__(
        self, locale: Locale = Locale.DEFAULT, seed: Optional[Seed] = None
    ) -> None:
        """Initialize attributes lazily."""
        super().__init__(seed=seed)
        self.locale = locale

        for provider in self._DEFAULT_PROVIDERS:
            name = getattr(provider.Meta, "name")  # type: ignore

            # Check if a provider is locale-dependent.
            if issubclass(provider, BaseDataProvider):
                setattr(self, f"_{name}", provider)
            elif issubclass(provider, BaseProvider):
                setattr(self, name, provider(seed=self.seed))

    class Meta:
        """Class for metadata."""

        name: Final[str] = "generic"

    def __getattr__(self, attrname: str) -> Any:
        """Get attribute without underscore.

        :param attrname: Attribute name.
        :return: An attribute.
        """
        attribute = object.__getattribute__(self, "_" + attrname)
        if attribute and callable(attribute):
            self.__dict__[attrname] = attribute(
                self.locale,
                self.seed,
            )
            return self.__dict__[attrname]

    def __dir__(self) -> List[str]:
        """Available data providers.

        The list of result will be used in AbstractField to
        determine method's class.

        :return: List of attributes.
        """
        attributes = []
        exclude = BaseProvider().__dict__.keys()

        for a in self.__dict__:
            if a not in exclude:
                if a.startswith("_"):
                    attribute = a.replace("_", "", 1)
                    attributes.append(attribute)
                else:
                    attributes.append(a)
        return attributes

    def add_provider(self, cls: Type[BaseProvider], **kwargs: Any) -> None:
        """Add a custom provider to Generic() object.

        :param cls: Custom provider.
        :return: None
        :raises TypeError: if cls is not class or is not a subclass
            of BaseProvider.
        """
        if inspect.isclass(cls):
            if not issubclass(cls, BaseProvider):
                raise TypeError(
                    "The provider must be a "
                    "subclass of mimesis.providers.BaseProvider"
                )
            try:
                name = cls.Meta.name  # type: ignore
            except AttributeError:
                name = cls.__name__.lower()

            if "seed" in kwargs:
                kwargs.pop("seed")

            setattr(self, name, cls(seed=self.seed, **kwargs))
        else:
            raise TypeError("The provider must be a class")

    def add_providers(self, *providers: Type[BaseProvider]) -> None:
        """Add a lot of custom providers to Generic() object.

        :param providers: Custom providers.
        :return: None
        """
        for provider in providers:
            self.add_provider(provider)

    def __str__(self) -> str:
        """Human-readable representation of locale."""
        return "{} <{}>".format(self.__class__.__name__, self.locale)



# -*- coding: utf-8 -*-

"""Provides all the data related to the internet."""
from typing import List

HTTP_METHODS = [
    "GET",
    "POST",
    "PUT",
    "DELETE",
    "OPTIONS",
    "CONNECT",
    "HEAD",
]

HTTP_STATUS_CODES = [
    100,
    101,
    102,
    103,
    200,
    201,
    202,
    203,
    204,
    205,
    206,
    207,
    208,
    226,
    300,
    301,
    302,
    303,
    304,
    305,
    306,
    307,
    308,
    400,
    401,
    402,
    403,
    404,
    405,
    406,
    407,
    408,
    409,
    410,
    411,
    412,
    413,
    414,
    415,
    416,
    417,
    421,
    422,
    423,
    424,
    426,
    428,
    429,
    431,
    451,
    500,
    501,
    502,
    503,
    504,
    505,
    506,
    507,
    508,
    510,
    511,
]

HTTP_STATUS_MSGS = [
    "100 Continue",
    "101 Switching Protocols",
    "102 Processing",
    "103 Early Hints",
    "200 OK",
    "201 Created",
    "202 Accepted",
    "203 Non Authoritative Information",
    "204 No Content",
    "205 Reset Content",
    "206 Partial Content",
    "207 Multi Status",
    "208 Already Reported",
    "226 IM Used",
    "300 Multiple Choices",
    "301 Moved Permanently",
    "302 Found",
    "303 See Other",
    "304 Not Modified",
    "305 Use Proxy",
    "306 Reserved",
    "307 Temporary Redirect",
    "308 Permanent Redirect",
    "400 Bad Request",
    "401 Unauthorized",
    "402 Payment Required",
    "403 Forbidden",
    "404 Not Found",
    "405 Method Not Allowed",
    "406 Not Acceptable",
    "407 Proxy Authentication Required",
    "408 Request Timeout",
    "409 Conflict",
    "410 Gone",
    "411 Length Required",
    "412 Precondition Failed",
    "413 Request Entity Too Large",
    "414 Request URI Too Long",
    "415 Unsupported Media Type",
    "416 Requested Range Not Satisfiable",
    "417 Expectation Failed",
    "421 Misdirected Request",
    "422 Unprocessable Entity",
    "423 Locked",
    "424 Failed Dependency",
    "426 Upgrade Required",
    "428 Precondition Required",
    "429 Too Many Requests",
    "431 Request Header Fields Too Large",
    "451 Unavailable For Legal Reasons",
    "500 Internal Server Error",
    "501 Not Implemented",
    "502 Bad Gateway",
    "503 Service Unavailable",
    "504 Gateway Timeout",
    "505 HTTP Version Not Supported",
    "506 Variant Also Negotiates",
    "507 Insufficient Storage",
    "508 Loop Detected",
    "510 Not Extended",
    "511 Network Authentication Required",
]

TLD = {
    "cctld": [
        ".ac",
        ".ad",
        ".ae",
        ".af",
        ".ag",
        ".ai",
        ".al",
        ".am",
        ".an",
        ".ao",
        ".aq",
        ".ar",
        ".as",
        ".at",
        ".au",
        ".aw",
        ".ax",
        ".az",
        ".ba",
        ".bb",
        ".bd",
        ".be",
        ".bf",
        ".bg",
        ".bh",
        ".bi",
        ".bj",
        ".bm",
        ".bn",
        ".bo",
        ".bq",
        ".br",
        ".bs",
        ".bt",
        ".bv",
        ".bw",
        ".by",
        ".bz",
        ".bzh",
        ".ca",
        ".cc",
        ".cd",
        ".cf",
        ".cg",
        ".ch",
        ".ci",
        ".ck",
        ".cl",
        ".cm",
        ".cn",
        ".co",
        ".cr",
        ".cu",
        ".cv",
        ".cw",
        ".cx",
        ".cy",
        ".cz",
        ".dd",
        ".de",
        ".dj",
        ".dk",
        ".dm",
        ".do",
        ".dz",
        ".ec",
        ".ee",
        ".eg",
        ".eh",
        ".er",
        ".es",
        ".et",
        ".eu",
        ".fi",
        ".fj",
        ".fk",
        ".fm",
        ".fo",
        ".fr",
        ".ga",
        ".gb",
        ".gd",
        ".ge",
        ".gf",
        ".gg",
        ".gh",
        ".gi",
        ".gl",
        ".gm",
        ".gn",
        ".gp",
        ".gq",
        ".gr",
        ".gsla",
        ".gt",
        ".gu",
        ".gw",
        ".gy",
        ".hk",
        ".hm",
        ".hn",
        ".hr",
        ".ht",
        ".hu",
        ".id",
        ".ie",
        ".il",
        ".im",
        ".in",
        ".io",
        ".iq",
        ".ir",
        ".is",
        ".it",
        ".je",
        ".jm",
        ".jo",
        ".jp",
        ".ke",
        ".kg",
        ".kh",
        ".ki",
        ".km",
        ".kn",
        ".kp",
        ".kr",
        ".krd",
        ".kw",
        ".ky",
        ".kz",
        ".la",
        ".lb",
        ".lc",
        ".li",
        ".lk",
        ".lr",
        ".ls",
        ".lt",
        ".lu",
        ".lv",
        ".ly",
        ".ma",
        ".mc",
        ".md",
        ".me",
        ".mg",
        ".mh",
        ".mk",
        ".ml",
        ".mm",
        ".mn",
        ".mo",
        ".mp",
        ".mq",
        ".mr",
        ".ms",
        ".mt",
        ".mu",
        ".mv",
        ".mw",
        ".mx",
        ".my",
        ".mz",
        ".na",
        ".nc",
        ".ne",
        ".nf",
        ".ng",
        ".ni",
        ".nl",
        ".no",
        ".np",
        ".nr",
        ".nu",
        ".nz",
        ".om",
        ".pa",
        ".pe",
        ".pf",
        ".pg",
        ".ph",
        ".pk",
        ".pl",
        ".pm",
        ".pn",
        ".pr",
        ".ps",
        ".pt",
        ".pw",
        ".py",
        ".qa",
        ".re",
        ".ro",
        ".rs",
        ".ru",
        ".rw",
        ".sa",
        ".sb",
        ".sc",
        ".sd",
        ".se",
        ".sg",
        ".sh",
        ".si",
        ".sk",
        ".sl",
        ".sm",
        ".sn",
        ".so",
        ".sr",
        ".ss",
        ".st",
        ".su",
        ".sv",
        ".sx",
        ".sy",
        ".sz",
        ".tc",
        ".td",
        ".tf",
        ".tg",
        ".th",
        ".tj",
        ".tk",
        ".tl",
        ".tm",
        ".tn",
        ".to",
        ".tp",
        ".tr",
        ".tt",
        ".tv",
        ".tw",
        ".tz",
        ".ua",
        ".ug",
        ".uk",
        ".us",
        ".uy",
        ".uz",
        ".va",
        ".vc",
        ".ve",
        ".vg",
        ".vi",
        ".vn",
        ".vu",
        ".wf",
        ".ws",
        ".ye",
        ".yt",
        ".yu",
        ".za",
        ".zm",
        ".zr",
        ".zw",
    ],
    "gtld": [
        ".academy",
        ".accountant",
        ".accountants",
        ".actor",
        ".ads",
        ".adult",
        ".agency",
        ".airforce",
        ".alsace",
        ".amsterdam",
        ".analytics",
        ".and",
        ".antivirus",
        ".apartments",
        ".app",
        ".aquitaine",
        ".archi",
        ".architect",
        ".are",
        ".army",
        ".art",
        ".associates",
        ".attorney",
        ".auction",
        ".audio",
        ".auto",
        ".autoinsurance",
        ".autos",
        ".baby",
        ".band",
        ".bank",
        ".bar",
        ".barcelona",
        ".bargains",
        ".baseball",
        ".basketball",
        ".bayern",
        ".beauty",
        ".beer",
        ".berlin",
        ".bet",
        ".bible",
        ".bid",
        ".bike",
        ".bingo",
        ".bio",
        ".black",
        ".blackfriday",
        ".blog",
        ".blue",
        ".boats",
        ".boo",
        ".book",
        ".boston",
        ".boutique",
        ".broadway",
        ".broker",
        ".brussels",
        ".budapest",
        ".build",
        ".builders",
        ".business",
        ".buy",
        ".buzz",
        ".cab",
        ".cafe",
        ".cal",
        ".cam",
        ".camera",
        ".camp",
        ".capital",
        ".car",
        ".cards",
        ".care",
        ".career",
        ".careers",
        ".carinsurance",
        ".cars",
        ".case",
        ".cash",
        ".casino",
        ".catalonia",
        ".catering",
        ".catholic",
        ".center",
        ".charity",
        ".chat",
        ".cheap",
        ".christmas",
        ".church",
        ".city",
        ".claims",
        ".cleaning",
        ".click",
        ".clinic",
        ".clothing",
        ".cloud",
        ".club",
        ".coach",
        ".codes",
        ".coffee",
        ".college",
        ".cologne",
        ".community",
        ".company",
        ".compare",
        ".computer",
        ".condos",
        ".construction",
        ".consulting",
        ".contact",
        ".contractors",
        ".cooking",
        ".cool",
        ".corp",
        ".country",
        ".coupon",
        ".coupons",
        ".courses",
        ".cpa",
        ".credit",
        ".creditcard",
        ".creditunion",
        ".cricket",
        ".cruise",
        ".cruises",
        ".cymru",
        ".dad",
        ".dance",
        ".data",
        ".date",
        ".dating",
        ".day",
        ".dds",
        ".deal",
        ".dealer",
        ".deals",
        ".degree",
        ".delivery",
        ".democrat",
        ".dental",
        ".dentist",
        ".design",
        ".dev",
        ".diamonds",
        ".diet",
        ".digital",
        ".direct",
        ".directory",
        ".discount",
        ".diy",
        ".docs",
        ".doctor",
        ".dog",
        ".domains",
        ".dot",
        ".download",
        ".drive",
        ".duck",
        ".earth",
        ".eat",
        ".eco",
        ".ecom",
        ".education",
        ".email",
        ".energy",
        ".engineer",
        ".engineering",
        ".enterprises",
        ".equipment",
        ".esq",
        ".estate",
        ".events",
        ".exchange",
        ".expert",
        ".exposed",
        ".express",
        ".fail",
        ".faith",
        ".family",
        ".fan",
        ".fans",
        ".farm",
        ".fashion",
        ".feedback",
        ".film",
        ".finance",
        ".financial",
        ".financialaid",
        ".fish",
        ".fishing",
        ".fit",
        ".fitness",
        ".flights",
        ".florist",
        ".flowers",
        ".fly",
        ".foo",
        ".food",
        ".football",
        ".forex",
        ".forsale",
        ".forum",
        ".foundation",
        ".free",
        ".frl",
        ".frontdoor",
        ".fun",
        ".fund",
        ".furniture",
        ".futbol",
        ".fyi",
        ".gallery",
        ".game",
        ".games",
        ".garden",
        ".gay",
        ".gent",
        ".gift",
        ".gifts",
        ".gives",
        ".giving",
        ".glass",
        ".glean",
        ".global",
        ".gmbh",
        ".gold",
        ".golf",
        ".gop",
        ".graphics",
        ".green",
        ".gripe",
        ".grocery",
        ".group",
        ".guide",
        ".guitars",
        ".guru",
        ".hair",
        ".halal",
        ".hamburg",
        ".hangout",
        ".health",
        ".healthcare",
        ".heart",
        ".help",
        ".helsinki",
        ".here",
        ".hiphop",
        ".hiv",
        ".hockey",
        ".holdings",
        ".holiday",
        ".home",
        ".homes",
        ".horse",
        ".hospital",
        ".host",
        ".hosting",
        ".hot",
        ".hoteis",
        ".hotel",
        ".hoteles",
        ".hotels",
        ".house",
        ".how",
        ".immo",
        ".inc",
        ".industries",
        ".ing",
        ".ink",
        ".institute",
        ".insurance",
        ".insure",
        ".international",
        ".investments",
        ".islam",
        ".jewelry",
        ".juegos",
        ".kid",
        ".kids",
        ".kitchen",
        ".koeln",
        ".kosher",
        ".land",
        ".law",
        ".lawyer",
        ".lds",
        ".lease",
        ".legal",
        ".lgbt",
        ".life",
        ".lifeinsurance",
        ".lifestyle",
        ".lighting",
        ".limited",
        ".limo",
        ".link",
        ".live",
        ".living",
        ".llc",
        ".llp",
        ".loan",
        ".loans",
        ".loft",
        ".lol",
        ".london",
        ".lotto",
        ".love",
        ".ltd",
        ".luxe",
        ".luxury",
        ".madrid",
        ".mail",
        ".makeup",
        ".man",
        ".management",
        ".map",
        ".market",
        ".marketing",
        ".markets",
        ".mba",
        ".med",
        ".media",
        ".medical",
        ".meet",
        ".meme",
        ".memorial",
        ".men",
        ".menu",
        ".miami",
        ".mls",
        ".mobile",
        ".mobily",
        ".mom",
        ".money",
        ".mormon",
        ".mortgage",
        ".moscow",
        ".motorcycles",
        ".mov",
        ".movie",
        ".music",
        ".mutualfunds",
        ".navy",
        ".network",
        ".new",
        ".news",
        ".ngo",
        ".ninja",
        ".now",
        ".nrw",
        ".nyc",
        ".one",
        ".ong",
        ".online",
        ".organic",
        ".page",
        ".paris",
        ".partners",
        ".parts",
        ".party",
        ".patch",
        ".pay",
        ".pet",
        ".pets",
        ".pharmacy",
        ".phd",
        ".phone",
        ".photo",
        ".photography",
        ".photos",
        ".physio",
        ".pics",
        ".pictures",
        ".pid",
        ".ping",
        ".pink",
        ".pizza",
        ".place",
        ".play",
        ".plumbing",
        ".plus",
        ".poker",
        ".porn",
        ".press",
        ".prod",
        ".productions",
        ".prof",
        ".promo",
        ".properties",
        ".property",
        ".protection",
        ".pub",
        ".qpon",
        ".quebec",
        ".racing",
        ".radio",
        ".realestate",
        ".realtor",
        ".realty",
        ".recipes",
        ".red",
        ".rehab",
        ".rent",
        ".rentals",
        ".repair",
        ".report",
        ".republican",
        ".restaurant",
        ".retirement",
        ".review",
        ".reviews",
        ".rip",
        ".rocks",
        ".rodeo",
        ".roma",
        ".room",
        ".rsvp",
        ".rugby",
        ".run",
        ".saarland",
        ".safety",
        ".sale",
        ".salon",
        ".save",
        ".scholarships",
        ".school",
        ".science",
        ".scot",
        ".search",
        ".secure",
        ".security",
        ".seek",
        ".services",
        ".sex",
        ".sexy",
        ".shiksha",
        ".shoes",
        ".shop",
        ".shopping",
        ".show",
        ".silk",
        ".singles",
        ".site",
        ".ski",
        ".skin",
        ".smile",
        ".soccer",
        ".social",
        ".software",
        ".solar",
        ".solutions",
        ".spa",
        ".space",
        ".sport",
        ".sports",
        ".spot",
        ".spreadbetting",
        ".star",
        ".stockholm",
        ".storage",
        ".store",
        ".stroke",
        ".studio",
        ".study",
        ".style",
        ".sucks",
        ".supplies",
        ".supply",
        ".support",
        ".surf",
        ".surgery",
        ".systems",
        ".talk",
        ".tattoo",
        ".tax",
        ".taxi",
        ".team",
        ".tech",
        ".technology",
        ".tennis",
        ".theater",
        ".theatre",
        ".tickets",
        ".tips",
        ".tires",
        ".tirol",
        ".today",
        ".tools",
        ".top",
        ".tour",
        ".tours",
        ".town",
        ".toys",
        ".trade",
        ".trading",
        ".training",
        ".tube",
        ".university",
        ".vacations",
        ".vegas",
        ".ventures",
        ".vet",
        ".video",
        ".villas",
        ".vin",
        ".vip",
        ".vision",
        ".vlaanderen",
        ".vodka",
        ".vote",
        ".voting",
        ".voyage",
        ".wales",
        ".watch",
        ".watches",
        ".weather",
        ".web",
        ".webcam",
        ".webs",
        ".website",
        ".wed",
        ".wedding",
        ".wien",
        ".wiki",
        ".win",
        ".wine",
        ".winners",
        ".work",
        ".works",
        ".world",
        ".wow",
        ".wtf",
        ".xyz",
        ".yachts",
        ".yoga",
        ".you",
        ".zip",
        ".zone",
        ".zuerich",
        ".zulu",
    ],
    "geotld": [
        ".abudhabi",
        ".africa",
        ".africa",
        ".alsace",
        ".amsterdam",
        ".aquitaine",
        ".bar",
        ".bar",
        ".barcelona",
        ".bayern",
        ".berlin",
        ".boston",
        ".brussels",
        ".budapest",
        ".capetown",
        ".catalonia",
        ".cologne",
        ".doha",
        ".dubai",
        ".durban",
        ".frl",
        ".gent",
        ".hamburg",
        ".helsinki",
        ".ist",
        ".istanbul",
        ".joburg",
        ".koeln",
        ".kyoto",
        ".london",
        ".madrid",
        ".melbourne",
        ".miami",
        ".moscow",
        ".nagoya",
        ".nrw",
        ".nyc",
        ".okinawa",
        ".osaka",
        ".osaka",
        ".paris",
        ".quebec",
        ".rio",
        ".saarland",
        ".scot",
        ".stockholm",
        ".sydney",
        ".taipei",
        ".tata",
        ".tirol",
        ".tokyo",
        ".tui",
        ".vlaanderen",
        ".wales",
        ".wien",
        ".yokohama",
        ".zuerich",
        ".zulu",
    ],
    "utld": [
        ".com",
        ".org",
        ".net",
        ".biz",
        ".info",
        ".name",
    ],
    "stld": [
        ".aero",
        ".asia",
        ".cat",
        ".coop",
        ".edu",
        ".gov",
        ".int",
        ".jobs",
        ".mil",
        ".mobi",
        ".museum.post",
        ".tel",
        ".travel",
        ".xxx",
    ],
}

EMAIL_DOMAINS: List[str] = [
    "@gmail.com",
    "@yandex.com",
    "@yahoo.com",
    "@live.com",
    "@outlook.com",
    "@protonmail.com",
]

EMOJI = (
    ":bowtie:",
    ":smile:",
    ":laughing:",
    ":blush:",
    ":smiley:",
    ":relaxed:",
    ":smirk:",
    ":heart_eyes:",
    ":kissing_heart:",
    ":kissing_closed_eyes:",
    ":flushed:",
    ":relieved:",
    ":satisfied:",
    ":grin:",
    ":wink:",
    ":stuck_out_tongue_winking_eye:",
    ":stuck_out_tongue_closed_eyes:",
    ":grinning:",
    ":kissing:",
    ":kissing_smiling_eyes:",
    ":stuck_out_tongue:",
    ":sleeping:",
    ":worried:",
    ":frowning:",
    ":anguished:",
    ":open_mouth:",
    ":grimacing:",
    ":confused:",
    ":hushed:",
    ":expressionless:",
    ":unamused:",
    ":sweat_smile:",
    ":sweat:",
    ":disappointed_relieved:",
    ":weary:",
    ":pensive:",
    ":disappointed:",
    ":confounded:",
    ":fearful:",
    ":cold_sweat:",
    ":persevere:",
    ":cry:",
    ":sob:",
    ":joy:",
    ":astonished:",
    ":scream:",
    ":neckbeard:",
    ":tired_face:",
    ":angry:",
    ":rage:",
    ":triumph:",
    ":sleepy:",
    ":yum:",
    ":mask:",
    ":sunglasses:",
    ":dizzy_face:",
    ":imp:",
    ":smiling_imp:",
    ":neutral_face:",
    ":no_mouth:",
    ":innocent:",
    ":alien:",
    ":yellow_heart:",
    ":blue_heart:",
    ":purple_heart:",
    ":heart:",
    ":green_heart:",
    ":broken_heart:",
    ":heartbeat:",
    ":heartpulse:",
    ":two_hearts:",
    ":revolving_hearts:",
    ":cupid:",
    ":sparkling_heart:",
    ":sparkles:",
    ":star:",
    ":star2:",
    ":dizzy:",
    ":boom:",
    ":collision:",
    ":anger:",
    ":exclamation:",
    ":question:",
    ":grey_exclamation:",
    ":grey_question:",
    ":zzz:",
    ":dash:",
    ":sweat_drops:",
    ":notes:",
    ":musical_note:",
    ":fire:",
    ":hankey:",
    ":poop:",
    ":shit:",
    ":+1:",
    ":thumbsup:",
    ":-1:",
    ":thumbsdown:",
    ":ok_hand:",
    ":punch:",
    ":facepunch:",
    ":fist:",
    ":v:",
    ":wave:",
    ":hand:",
    ":raised_hand:",
    ":open_hands:",
    ":point_up:",
    ":point_down:",
    ":point_left:",
    ":point_right:",
    ":raised_hands:",
    ":pray:",
    ":point_up_2:",
    ":clap:",
    ":muscle:",
    ":metal:",
    ":fu:",
    ":runner:",
    ":running:",
    ":couple:",
    ":family:",
    ":two_men_holding_hands:",
    ":two_women_holding_hands:",
    ":dancer:",
    ":dancers:",
    ":ok_woman:",
    ":no_good:",
    ":information_desk_person:",
    ":raising_hand:",
    ":bride_with_veil:",
    ":person_with_pouting_face:",
    ":person_frowning:",
    ":bow:",
    ":couplekiss:",
    ":couple_with_heart:",
    ":massage:",
    ":haircut:",
    ":nail_care:",
    ":boy:",
    ":girl:",
    ":woman:",
    ":man:",
    ":baby:",
    ":older_woman:",
    ":older_man:",
    ":person_with_blond_hair:",
    ":man_with_gua_pi_mao:",
    ":man_with_turban:",
    ":construction_worker:",
    ":cop:",
    ":angel:",
    ":princess:",
    ":smiley_cat:",
    ":smile_cat:",
    ":heart_eyes_cat:",
    ":kissing_cat:",
    ":smirk_cat:",
    ":scream_cat:",
    ":crying_cat_face:",
    ":joy_cat:",
    ":pouting_cat:",
    ":japanese_ogre:",
    ":japanese_goblin:",
    ":see_no_evil:",
    ":hear_no_evil:",
    ":speak_no_evil:",
    ":guardsman:",
    ":skull:",
    ":feet:",
    ":lips:",
    ":kiss:",
    ":droplet:",
    ":ear:",
    ":eyes:",
    ":nose:",
    ":tongue:",
    ":love_letter:",
    ":bust_in_silhouette:",
    ":busts_in_silhouette:",
    ":speech_balloon:",
    ":thought_balloon:",
    ":feelsgood:",
    ":finnadie:",
    ":goberserk:",
    ":godmode:",
    ":hurtrealbad:",
    ":rage1:",
    ":rage2:",
    ":rage3:",
    ":rage4:",
    ":suspect:",
    ":trollface:",
    ":sunny:",
    ":umbrella:",
    ":cloud:",
    ":snowflake:",
    ":snowman:",
    ":zap:",
    ":cyclone:",
    ":foggy:",
    ":ocean:",
    ":cat:",
    ":dog:",
    ":mouse:",
    ":hamster:",
    ":rabbit:",
    ":wolf:",
    ":frog:",
    ":tiger:",
    ":koala:",
    ":bear:",
    ":pig:",
    ":pig_nose:",
    ":cow:",
    ":boar:",
    ":monkey_face:",
    ":monkey:",
    ":horse:",
    ":racehorse:",
    ":camel:",
    ":sheep:",
    ":elephant:",
    ":panda_face:",
    ":snake:",
    ":bird:",
    ":baby_chick:",
    ":hatched_chick:",
    ":hatching_chick:",
    ":chicken:",
    ":penguin:",
    ":turtle:",
    ":bug:",
    ":honeybee:",
    ":ant:",
    ":beetle:",
    ":snail:",
    ":octopus:",
    ":tropical_fish:",
    ":fish:",
    ":whale:",
    ":whale2:",
    ":dolphin:",
    ":cow2:",
    ":ram:",
    ":rat:",
    ":water_buffalo:",
    ":tiger2:",
    ":rabbit2:",
    ":dragon:",
    ":goat:",
    ":rooster:",
    ":dog2:",
    ":pig2:",
    ":mouse2:",
    ":ox:",
    ":dragon_face:",
    ":blowfish:",
    ":crocodile:",
    ":dromedary_camel:",
    ":leopard:",
    ":cat2:",
    ":poodle:",
    ":paw_prints:",
    ":bouquet:",
    ":cherry_blossom:",
    ":tulip:",
    ":four_leaf_clover:",
    ":rose:",
    ":sunflower:",
    ":hibiscus:",
    ":maple_leaf:",
    ":leaves:",
    ":fallen_leaf:",
    ":herb:",
    ":mushroom:",
    ":cactus:",
    ":palm_tree:",
    ":evergreen_tree:",
    ":deciduous_tree:",
    ":chestnut:",
    ":seedling:",
    ":blossom:",
    ":ear_of_rice:",
    ":shell:",
    ":globe_with_meridians:",
    ":sun_with_face:",
    ":full_moon_with_face:",
    ":new_moon_with_face:",
    ":new_moon:",
    ":waxing_crescent_moon:",
    ":first_quarter_moon:",
    ":waxing_gibbous_moon:",
    ":full_moon:",
    ":waning_gibbous_moon:",
    ":last_quarter_moon:",
    ":waning_crescent_moon:",
    ":last_quarter_moon_with_face:",
    ":first_quarter_moon_with_face:",
    ":crescent_moon:",
    ":earth_africa:",
    ":earth_americas:",
    ":earth_asia:",
    ":volcano:",
    ":milky_way:",
    ":partly_sunny:",
    ":octocat:",
    ":squirrel:",
    ":bamboo:",
    ":gift_heart:",
    ":dolls:",
    ":school_satchel:",
    ":mortar_board:",
    ":flags:",
    ":fireworks:",
    ":sparkler:",
    ":wind_chime:",
    ":rice_scene:",
    ":jack_o_lantern:",
    ":ghost:",
    ":santa:",
    ":christmas_tree:",
    ":gift:",
    ":bell:",
    ":no_bell:",
    ":tanabata_tree:",
    ":tada:",
    ":confetti_ball:",
    ":balloon:",
    ":crystal_ball:",
    ":cd:",
    ":dvd:",
    ":floppy_disk:",
    ":camera:",
    ":video_camera:",
    ":movie_camera:",
    ":computer:",
    ":tv:",
    ":iphone:",
    ":phone:",
    ":telephone:",
    ":telephone_receiver:",
    ":pager:",
    ":fax:",
    ":minidisc:",
    ":vhs:",
    ":sound:",
    ":speaker:",
    ":mute:",
    ":loudspeaker:",
    ":mega:",
    ":hourglass:",
    ":hourglass_flowing_sand:",
    ":alarm_clock:",
    ":watch:",
    ":radio:",
    ":satellite:",
    ":loop:",
    ":mag:",
    ":mag_right:",
    ":unlock:",
    ":lock:",
    ":lock_with_ink_pen:",
    ":closed_lock_with_key:",
    ":key:",
    ":bulb:",
    ":flashlight:",
    ":high_brightness:",
    ":low_brightness:",
    ":electric_plug:",
    ":battery:",
    ":calling:",
    ":email:",
    ":mailbox:",
    ":postbox:",
    ":bath:",
    ":bathtub:",
    ":shower:",
    ":toilet:",
    ":wrench:",
    ":nut_and_bolt:",
    ":hammer:",
    ":seat:",
    ":moneybag:",
    ":yen:",
    ":dollar:",
    ":pound:",
    ":euro:",
    ":credit_card:",
    ":money_with_wings:",
    ":e-mail:",
    ":inbox_tray:",
    ":outbox_tray:",
    ":envelope:",
    ":incoming_envelope:",
    ":postal_horn:",
    ":mailbox_closed:",
    ":mailbox_with_mail:",
    ":mailbox_with_no_mail:",
    ":package:",
    ":door:",
    ":smoking:",
    ":bomb:",
    ":gun:",
    ":hocho:",
    ":pill:",
    ":syringe:",
    ":page_facing_up:",
    ":page_with_curl:",
    ":bookmark_tabs:",
    ":bar_chart:",
    ":chart_with_upwards_trend:",
    ":chart_with_downwards_trend:",
    ":scroll:",
    ":clipboard:",
    ":calendar:",
    ":date:",
    ":card_index:",
    ":file_folder:",
    ":open_file_folder:",
    ":scissors:",
    ":pushpin:",
    ":paperclip:",
    ":black_nib:",
    ":pencil2:",
    ":straight_ruler:",
    ":triangular_ruler:",
    ":closed_book:",
    ":green_book:",
    ":blue_book:",
    ":orange_book:",
    ":notebook:",
    ":notebook_with_decorative_cover:",
    ":ledger:",
    ":books:",
    ":bookmark:",
    ":name_badge:",
    ":microscope:",
    ":telescope:",
    ":newspaper:",
    ":football:",
    ":basketball:",
    ":soccer:",
    ":baseball:",
    ":tennis:",
    ":8ball:",
    ":rugby_football:",
    ":bowling:",
    ":golf:",
    ":mountain_bicyclist:",
    ":bicyclist:",
    ":horse_racing:",
    ":snowboarder:",
    ":swimmer:",
    ":surfer:",
    ":ski:",
    ":spades:",
    ":hearts:",
    ":clubs:",
    ":diamonds:",
    ":gem:",
    ":ring:",
    ":trophy:",
    ":musical_score:",
    ":musical_keyboard:",
    ":violin:",
    ":space_invader:",
    ":video_game:",
    ":black_joker:",
    ":flower_playing_cards:",
    ":game_die:",
    ":dart:",
    ":mahjong:",
    ":clapper:",
    ":memo:",
    ":pencil:",
    ":book:",
    ":art:",
    ":microphone:",
    ":headphones:",
    ":trumpet:",
    ":saxophone:",
    ":guitar:",
    ":shoe:",
    ":sandal:",
    ":high_heel:",
    ":lipstick:",
    ":boot:",
    ":shirt:",
    ":tshirt:",
    ":necktie:",
    ":womans_clothes:",
    ":dress:",
    ":running_shirt_with_sash:",
    ":jeans:",
    ":kimono:",
    ":bikini:",
    ":ribbon:",
    ":tophat:",
    ":crown:",
    ":womans_hat:",
    ":mans_shoe:",
    ":closed_umbrella:",
    ":briefcase:",
    ":handbag:",
    ":pouch:",
    ":purse:",
    ":eyeglasses:",
    ":fishing_pole_and_fish:",
    ":coffee:",
    ":tea:",
    ":sake:",
    ":baby_bottle:",
    ":beer:",
    ":beers:",
    ":cocktail:",
    ":tropical_drink:",
    ":wine_glass:",
    ":fork_and_knife:",
    ":pizza:",
    ":hamburger:",
    ":fries:",
    ":poultry_leg:",
    ":meat_on_bone:",
    ":spaghetti:",
    ":curry:",
    ":fried_shrimp:",
    ":bento:",
    ":sushi:",
    ":fish_cake:",
    ":rice_ball:",
    ":rice_cracker:",
    ":rice:",
    ":ramen:",
    ":stew:",
    ":oden:",
    ":dango:",
    ":egg:",
    ":bread:",
    ":doughnut:",
    ":custard:",
    ":icecream:",
    ":ice_cream:",
    ":shaved_ice:",
    ":birthday:",
    ":cake:",
    ":cookie:",
    ":chocolate_bar:",
    ":candy:",
    ":lollipop:",
    ":honey_pot:",
    ":apple:",
    ":green_apple:",
    ":tangerine:",
    ":lemon:",
    ":cherries:",
    ":grapes:",
    ":watermelon:",
    ":strawberry:",
    ":peach:",
    ":melon:",
    ":banana:",
    ":pear:",
    ":pineapple:",
    ":sweet_potato:",
    ":eggplant:",
    ":tomato:",
    ":corn:",
    ":house:",
    ":house_with_garden:",
    ":school:",
    ":office:",
    ":post_office:",
    ":hospital:",
    ":bank:",
    ":convenience_store:",
    ":love_hotel:",
    ":hotel:",
    ":wedding:",
    ":mimesis:",
    ":department_store:",
    ":european_post_office:",
    ":city_sunrise:",
    ":city_sunset:",
    ":japanese_castle:",
    ":european_castle:",
    ":tent:",
    ":factory:",
    ":tokyo_tower:",
    ":japan:",
    ":mount_fuji:",
    ":sunrise_over_mountains:",
    ":sunrise:",
    ":stars:",
    ":statue_of_liberty:",
    ":bridge_at_night:",
    ":carousel_horse:",
    ":rainbow:",
    ":ferris_wheel:",
    ":fountain:",
    ":roller_coaster:",
    ":ship:",
    ":speedboat:",
    ":boat:",
    ":sailboat:",
    ":rowboat:",
    ":anchor:",
    ":rocket:",
    ":airplane:",
    ":helicopter:",
    ":steam_locomotive:",
    ":tram:",
    ":mountain_railway:",
    ":bike:",
    ":aerial_tramway:",
    ":suspension_railway:",
    ":mountain_cableway:",
    ":tractor:",
    ":blue_car:",
    ":oncoming_automobile:",
    ":car:",
    ":red_car:",
    ":taxi:",
    ":oncoming_taxi:",
    ":articulated_lorry:",
    ":bus:",
    ":oncoming_bus:",
    ":rotating_light:",
    ":police_car:",
    ":oncoming_police_car:",
    ":fire_engine:",
    ":ambulance:",
    ":minibus:",
    ":truck:",
    ":train:",
    ":station:",
    ":train2:",
    ":bullettrain_front:",
    ":bullettrain_side:",
    ":light_rail:",
    ":monorail:",
    ":railway_car:",
    ":trolleybus:",
    ":ticket:",
    ":fuelpump:",
    ":vertical_traffic_light:",
    ":traffic_light:",
    ":warning:",
    ":construction:",
    ":beginner:",
    ":atm:",
    ":slot_machine:",
    ":busstop:",
    ":barber:",
    ":hotsprings:",
    ":checkered_flag:",
    ":crossed_flags:",
    ":izakaya_lantern:",
    ":moyai:",
    ":circus_tent:",
    ":performing_arts:",
    ":round_pushpin:",
    ":triangular_flag_on_post:",
    ":jp:",
    ":kr:",
    ":cn:",
    ":us:",
    ":fr:",
    ":es:",
    ":it:",
    ":ru:",
    ":gb:",
    ":uk:",
    ":de:",
    ":one:",
    ":two:",
    ":three:",
    ":four:",
    ":five:",
    ":six:",
    ":seven:",
    ":eight:",
    ":nine:",
    ":keycap_ten:",
    ":1234:",
    ":zero:",
    ":hash:",
    ":symbols:",
    ":arrow_backward:",
    ":arrow_down:",
    ":arrow_forward:",
    ":arrow_left:",
    ":capital_abcd:",
    ":abcd:",
    ":abc:",
    ":arrow_lower_left:",
    ":arrow_lower_right:",
    ":arrow_right:",
    ":arrow_up:",
    ":arrow_upper_left:",
    ":arrow_upper_right:",
    ":arrow_double_down:",
    ":arrow_double_up:",
    ":arrow_down_small:",
    ":arrow_heading_down:",
    ":arrow_heading_up:",
    ":leftwards_arrow_with_hook:",
    ":arrow_right_hook:",
    ":left_right_arrow:",
    ":arrow_up_down:",
    ":arrow_up_small:",
    ":arrows_clockwise:",
    ":arrows_counterclockwise:",
    ":rewind:",
    ":fast_forward:",
    ":information_source:",
    ":ok:",
    ":twisted_rightwards_arrows:",
    ":repeat:",
    ":repeat_one:",
    ":new:",
    ":top:",
    ":up:",
    ":cool:",
    ":free:",
    ":ng:",
    ":cinema:",
    ":koko:",
    ":signal_strength:",
    ":u5272:",
    ":u5408:",
    ":u55b6:",
    ":u6307:",
    ":u6708:",
    ":u6709:",
    ":u6e80:",
    ":u7121:",
    ":u7533:",
    ":u7a7a:",
    ":u7981:",
    ":sa:",
    ":restroom:",
    ":mens:",
    ":womens:",
    ":baby_symbol:",
    ":no_smoking:",
    ":parking:",
    ":wheelchair:",
    ":metro:",
    ":baggage_claim:",
    ":accept:",
    ":wc:",
    ":potable_water:",
    ":put_litter_in_its_place:",
    ":secret:",
    ":congratulations:",
    ":m:",
    ":passport_control:",
    ":left_luggage:",
    ":customs:",
    ":ideograph_advantage:",
    ":cl:",
    ":sos:",
    ":id:",
    ":no_entry_sign:",
    ":underage:",
    ":no_mobile_phones:",
    ":do_not_litter:",
    ":non-potable_water:",
    ":no_bicycles:",
    ":no_pedestrians:",
    ":children_crossing:",
    ":no_entry:",
    ":eight_spoked_asterisk:",
    ":sparkle:",
    ":eight_pointed_black_star:",
    ":heart_decoration:",
    ":vs:",
    ":vibration_mode:",
    ":mobile_phone_off:",
    ":chart:",
    ":currency_exchange:",
    ":aries:",
    ":taurus:",
    ":gemini:",
    ":cancer:",
    ":leo:",
    ":virgo:",
    ":libra:",
    ":scorpius:",
    ":sagittarius:",
    ":capricorn:",
    ":aquarius:",
    ":pisces:",
    ":ophiuchus:",
    ":six_pointed_star:",
    ":negative_squared_cross_mark:",
    ":a:",
    ":b:",
    ":ab:",
    ":o2:",
    ":diamond_shape_with_a_dot_inside:",
    ":recycle:",
    ":end:",
    ":back:",
    ":on:",
    ":soon:",
    ":clock1:",
    ":clock130:",
    ":clock10:",
    ":clock1030:",
    ":clock11:",
    ":clock1130:",
    ":clock12:",
    ":clock1230:",
    ":clock2:",
    ":clock230:",
    ":clock3:",
    ":clock330:",
    ":clock4:",
    ":clock430:",
    ":clock5:",
    ":clock530:",
    ":clock6:",
    ":clock630:",
    ":clock7:",
    ":clock730:",
    ":clock8:",
    ":clock830:",
    ":clock9:",
    ":clock930:",
    ":heavy_dollar_sign:",
    ":copyright:",
    ":registered:",
    ":tm:",
    ":x:",
    ":heavy_exclamation_mark:",
    ":bangbang:",
    ":interrobang:",
    ":o:",
    ":heavy_multiplication_x:",
    ":heavy_plus_sign:",
    ":heavy_minus_sign:",
    ":heavy_division_sign:",
    ":white_flower:",
    ":100:",
    ":heavy_check_mark:",
    ":ballot_box_with_check:",
    ":radio_button:",
    ":link:",
    ":curly_loop:",
    ":wavy_dash:",
    ":part_alternation_mark:",
    ":trident:",
    ":black_small_square:",
    ":white_small_square:",
    ":black_medium_small_square:",
    ":white_medium_small_square:",
    ":black_medium_square:",
    ":white_medium_square:",
    ":black_large_square:",
    ":white_large_square:",
    ":white_check_mark:",
    ":black_square_button:",
    ":white_square_button:",
    ":black_circle:",
    ":white_circle:",
    ":red_circle:",
    ":large_blue_circle:",
    ":large_blue_diamond:",
    ":large_orange_diamond:",
    ":small_blue_diamond:",
    ":small_orange_diamond:",
    ":small_red_triangle:",
    ":small_red_triangle_down:",
    ":shipit:",
)

USER_AGENTS = [
    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; de) Opera 8.0",
    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; de) Opera 8.02",
    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; en) Opera 8.0",
    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; en) Opera 8.02",
    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; en) Opera 8.52",
    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; en) Opera 8.53",
    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; en) Opera 8.54",
    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; pl) Opera 8.54",
    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; da) Opera 8.54",
    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; de) Opera 8.0",
    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; de) Opera 8.01",
    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; de) Opera 8.02",
    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; de) Opera 8.52",
    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; de) Opera 8.54",
    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; de) Opera 9.50",
    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera",
    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 7.60",
    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 8.0",
    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 8.00",
    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 8.01",
    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 8.02",
    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 8.52",
    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 8.53",
    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 8.54",
    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 9.24",
    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 9.26",
]



# -*- coding: utf-8 -*-

"""Here you can see data that used in all locales."""

from .address import *
from .code import *
from .common import *
from .datetime import *
from .development import *
from .file import *
from .finance import *
from .hardware import *
from .internet import *
from .path import *
from .payment import *
from .person import *
from .scientific import *
from .text import *
from .transport import *



# -*- coding: utf-8 -*-

"""Builtin specific data providers."""

from .da import DenmarkSpecProvider
from .en import USASpecProvider
from .it import ItalySpecProvider
from .nl import NetherlandsSpecProvider
from .pl import PolandSpecProvider
from .pt_br import BrazilSpecProvider
from .ru import RussiaSpecProvider
from .uk import UkraineSpecProvider

__all__ = [
    "USASpecProvider",
    "RussiaSpecProvider",
    "BrazilSpecProvider",
    "NetherlandsSpecProvider",
    "UkraineSpecProvider",
    "PolandSpecProvider",
    "DenmarkSpecProvider",
    "ItalySpecProvider",
]



# -*- coding: utf-8 -*-

"""This module is provide internal util functions."""

from string import ascii_letters, digits, punctuation
from typing import Union

from mimesis.data import COMMON_LETTERS, ROMANIZATION_DICT
from mimesis.locales import Locale, validate_locale

__all__ = ["romanize", "luhn_checksum"]


def luhn_checksum(num: str) -> str:
    """Calculate a checksum for num using the Luhn algorithm.

    :param num: The number to calculate a checksum for as a string.
    :return: Checksum for number.
    """
    check = 0
    for i, s in enumerate(reversed(num)):
        sx = int(s)
        sx = sx * 2 if i % 2 == 0 else sx
        sx = sx - 9 if sx > 9 else sx
        check += sx
    return str(check * 9 % 10)


def romanize(string: str, locale: Union[Locale, str]) -> str:
    """Romanize a given string.

    Supported locales are:
        Locale.RU
        Locale.UK
        Locale.KK

    :param string: Cyrillic string.
    :param locale: Locale.
    :return: Romanized string.
    """
    locale = validate_locale(locale)

    if locale not in (Locale.RU, Locale.UK, Locale.KK):
        raise ValueError(f"Romanization is not available for: {locale}")

    # Cyrillic string can contain ascii symbols, digits and punctuation.
    alphabet = {s: s for s in ascii_letters + digits + punctuation}
    alphabet.update(
        {
            **ROMANIZATION_DICT[locale.value],
            **COMMON_LETTERS,
        }
    )

    return "".join([alphabet[i] for i in string if i in alphabet])



# -*- coding: utf-8 -*-

"""Specific data provider for Italy (it)."""

import string
from typing import Final, Optional

from mimesis.builtins.base import BaseSpecProvider
from mimesis.enums import Gender
from mimesis.locales import Locale
from mimesis.typing import Seed

__all__ = ["ItalySpecProvider"]


class ItalySpecProvider(BaseSpecProvider):
    """Specific-provider of misc data for Italy."""

    def __init__(self, seed: Optional[Seed] = None) -> None:
        """Initialize attributes."""
        super().__init__(locale=Locale.IT, seed=seed)
        self._load_datafile(self._datafile)

    class Meta:
        """The name of the provider."""

        name: Final[str] = "italy_provider"

    def fiscal_code(self, gender: Optional[Gender] = None) -> str:
        """Return a random fiscal code.

        :param gender: Gender's enum object.
        :return: Fiscal code.

        Example:
            RSSMRA66R05D612U
        """
        code = "".join(self.random.choices(string.ascii_uppercase, k=6))

        code += self.random.custom_code(mask="##")

        month_codes = self.extract(["fiscal_code", "month_codes"])
        code += self.random.choice(month_codes)

        birth_day = self.random.randint(101, 131)
        self.validate_enum(gender, Gender)
        if gender == Gender.FEMALE:
            birth_day += 40
        code += str(birth_day)[1:]

        city_letters = self.extract(["fiscal_code", "city_letters"])
        code += self.random.choice(city_letters)
        code += self.random.custom_code(mask="###@")

        return code



# -*- coding: utf-8 -*-

"""Provides data related to food."""
from typing import Any, Final, List

from mimesis.providers.base import BaseDataProvider

__all__ = ["Food"]


class Food(BaseDataProvider):
    """Class for generating data related to food."""

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        """Initialize attributes.

        :param locale: Current locale.
        """
        super().__init__(*args, **kwargs)
        self._datafile = "food.json"
        self._load_datafile(self._datafile)

    class Meta:
        """Class for metadata."""

        name: Final[str] = "food"

    def _choice_from(self, key: str) -> str:
        """Choice random element."""
        data: List[str] = self.extract([key])
        return self.random.choice(data)

    def vegetable(self) -> str:
        """Get a random vegetable.

        :return: Vegetable name.

        :Example:
            Tomato.
        """
        return self._choice_from("vegetables")

    def fruit(self) -> str:
        """Get a random fruit or berry.

        :return: Fruit name.

        :Example:
            Banana.
        """
        return self._choice_from("fruits")

    def dish(self) -> str:
        """Get a random dish.

        :return: Dish name.

        :Example:
            Ratatouille.
        """
        return self._choice_from("dishes")

    def spices(self) -> str:
        """Get a random spices or herbs.

        :return: Spices or herbs.

        :Example:
            Anise.
        """
        return self._choice_from("spices")

    def drink(self) -> str:
        """Get a random drink.

        :return: Alcoholic drink.

        :Example:
            Vodka.
        """
        return self._choice_from("drinks")



# -*- coding: utf-8 -*-

"""Custom exceptions which used in Mimesis."""

from typing import Any, Optional, Union

from mimesis.enums import Locale


class LocaleError(ValueError):
    """Raised when a locale isn't supported."""

    def __init__(self, locale: Union[Locale, str]) -> None:
        """Initialize attributes for informative output.

        :param locale: Locale.
        """
        self.locale = locale

    def __str__(self) -> str:
        return f"Invalid locale «{self.locale}»"


class SchemaError(ValueError):
    """Raised when schema is unsupported."""

    def __str__(self) -> str:
        return "Schema should a callable object."


class NonEnumerableError(TypeError):
    """Raised when object is not instance of Enum."""

    message = "You should use one item of: «{}» of the object mimesis.enums.{}"

    def __init__(self, enum_obj: Any) -> None:
        """Initialize attributes for informative output.

        :param enum_obj: Enum object.
        """
        if enum_obj:
            self.name = enum_obj
            self.items = ", ".join([str(i) for i in enum_obj])
        else:
            self.items = ""

    def __str__(self) -> str:
        return self.message.format(self.items, self.name.__name__)


class FieldError(ValueError):
    def __init__(self, name: Optional[str] = None) -> None:
        """Initialize attributes for more informative output.

        :param name: Name of the field..
        """
        self.name = name
        self.message = "A field «{}» is not supported."
        self.message_none = "Field cannot be None."

    def __str__(self) -> str:
        if self.name is None:
            return self.message_none
        return self.message.format(self.name)



# -*- coding: utf-8 -*-

"""Mimesis is a Python library, which helps generate fake data.

Copyright (c) 2016 - 2021 Isaak Uchakaev (Likid Geimfari).
Website: https://mimesis.name
Email: <likid.geimfari@gmail.com>
Repository: https://github.com/lk-geimfari/mimesis
"""

from mimesis.schema import Field, Schema

from .providers import (
    Address,
    BaseDataProvider,
    BaseProvider,
    BinaryFile,
    Choice,
    Code,
    Cryptographic,
    Datetime,
    Development,
    File,
    Finance,
    Food,
    Generic,
    Hardware,
    Internet,
    Numeric,
    Path,
    Payment,
    Person,
    Science,
    Text,
    Transport,
)

__all__ = [
    "Address",
    "BaseDataProvider",
    "BaseProvider",
    "BinaryFile",
    "Finance",
    "Code",
    "Choice",
    "Datetime",
    "Development",
    "File",
    "Food",
    "Hardware",
    "Internet",
    "Numeric",
    "Path",
    "Payment",
    "Person",
    "Science",
    "Text",
    "Transport",
    "Cryptographic",
    # Has all:
    "Generic",
    # Schema:
    "Field",
    "Schema",
    # Meta:
    "__version__",
    "__title__",
    "__description__",
    "__url__",
    "__author__",
    "__author_email__",
    "__license__",
]

__version__ = "5.0.0"
__title__ = "mimesis"
__description__ = "Mimesis: fake data generator."
__url__ = "https://github.com/lk-geimfari/mimesis"
__author__ = "Isaak Uchakaev (Likid Geimfari)"
__author_email__ = "likid.geimfari@gmail.com"
__license__ = "MIT License"



# -*- coding: utf-8 -*-

"""Custom types and shortcuts for annotating Mimesis."""

import datetime
from decimal import Decimal
from typing import Any, Callable, Dict, List, Set, Tuple, Union

__all__ = [
    "Date",
    "DateTime",
    "JSON",
    "Matrix",
    "Seed",
    "Time",
    "Timestamp",
    "Keywords",
    "SchemaType",
]

JSON = Dict[str, Any]

DateTime = datetime.datetime

Time = datetime.time

Date = datetime.date

Timestamp = Union[str, int]

Seed = Union[int, str, bytes, bytearray]

Keywords = Union[List[str], Set[str], Tuple[str, ...]]

Matrix = Union[
    List[int],
    List[float],
    List[complex],
    List[Decimal],
]

SchemaType = Callable[[], JSON]
